from typing import Any

from .psycopg2 import PGDialect_psycopg2

class PGDialect_psycopg2cffi(PGDialect_psycopg2):
    driver: str
    supports_unicode_statements: bool
    supports_statement_cache: bool
    FEATURE_VERSION_MAP: Any
    @classmethod
    def dbapi(cls): ...

dialect = PGDialect_psycopg2cffi



from typing import Any

from .base import MySQLDialect

class MariaDBDialect(MySQLDialect):
    is_mariadb: bool
    supports_statement_cache: bool
    name: str
    preparer: Any

def loader(driver): ...



from typing import Any

from ...types import Float, Numeric
from .base import FBDialect, FBExecutionContext

class _kinterbasdb_numeric:
    def bind_processor(self, dialect): ...

class _FBNumeric_kinterbasdb(_kinterbasdb_numeric, Numeric): ...
class _FBFloat_kinterbasdb(_kinterbasdb_numeric, Float): ...

class FBExecutionContext_kinterbasdb(FBExecutionContext):
    @property
    def rowcount(self): ...

class FBDialect_kinterbasdb(FBDialect):
    driver: str
    supports_statement_cache: bool
    supports_sane_rowcount: bool
    supports_sane_multi_rowcount: bool
    supports_native_decimal: bool
    colspecs: Any
    enable_rowcount: Any
    type_conv: Any
    concurrency_level: Any
    retaining: Any
    def __init__(
        self, type_conv: int = ..., concurrency_level: int = ..., enable_rowcount: bool = ..., retaining: bool = ..., **kwargs
    ) -> None: ...
    @classmethod
    def dbapi(cls): ...
    def do_execute(self, cursor, statement, parameters, context: Any | None = ...) -> None: ...
    def do_rollback(self, dbapi_connection) -> None: ...
    def do_commit(self, dbapi_connection) -> None: ...
    def create_connect_args(self, url): ...
    def is_disconnect(self, e, connection, cursor): ...

dialect = FBDialect_kinterbasdb



from typing import Any

from ...engine import AdaptedConnection
from .pymysql import MySQLDialect_pymysql

class AsyncAdapt_asyncmy_cursor:
    server_side: bool
    await_: Any
    def __init__(self, adapt_connection) -> None: ...
    @property
    def description(self): ...
    @property
    def rowcount(self): ...
    @property
    def arraysize(self): ...
    @arraysize.setter
    def arraysize(self, value) -> None: ...
    @property
    def lastrowid(self): ...
    def close(self) -> None: ...
    def execute(self, operation, parameters: Any | None = ...): ...
    def executemany(self, operation, seq_of_parameters): ...
    def setinputsizes(self, *inputsizes) -> None: ...
    def __iter__(self): ...
    def fetchone(self): ...
    def fetchmany(self, size: Any | None = ...): ...
    def fetchall(self): ...

class AsyncAdapt_asyncmy_ss_cursor(AsyncAdapt_asyncmy_cursor):
    server_side: bool
    await_: Any
    def __init__(self, adapt_connection) -> None: ...
    def close(self) -> None: ...
    def fetchone(self): ...
    def fetchmany(self, size: Any | None = ...): ...
    def fetchall(self): ...

class AsyncAdapt_asyncmy_connection(AdaptedConnection):
    await_: Any
    dbapi: Any
    def __init__(self, dbapi, connection) -> None: ...
    def ping(self, reconnect): ...
    def character_set_name(self): ...
    def autocommit(self, value) -> None: ...
    def cursor(self, server_side: bool = ...): ...
    def rollback(self) -> None: ...
    def commit(self) -> None: ...
    def close(self) -> None: ...

class AsyncAdaptFallback_asyncmy_connection(AsyncAdapt_asyncmy_connection):
    await_: Any

class AsyncAdapt_asyncmy_dbapi:
    asyncmy: Any
    pymysql: Any
    paramstyle: str
    def __init__(self, asyncmy, pymysql) -> None: ...
    def connect(self, *arg, **kw): ...

class MySQLDialect_asyncmy(MySQLDialect_pymysql):
    driver: str
    supports_statement_cache: bool
    supports_server_side_cursors: bool
    is_async: bool
    @classmethod
    def dbapi(cls): ...
    @classmethod
    def get_pool_class(cls, url): ...
    def create_connect_args(self, url): ...
    def is_disconnect(self, e, connection, cursor): ...
    def get_driver_connection(self, connection): ...

dialect = MySQLDialect_asyncmy



from typing import Any

from sqlalchemy import types as sqltypes
from sqlalchemy.engine import default, reflection
from sqlalchemy.sql import compiler
from sqlalchemy.types import (
    BIGINT as BIGINT,
    BINARY as BINARY,
    CHAR as CHAR,
    DATE as DATE,
    DATETIME as DATETIME,
    DECIMAL as DECIMAL,
    FLOAT as FLOAT,
    INT as INT,
    INTEGER as INTEGER,
    NCHAR as NCHAR,
    NUMERIC as NUMERIC,
    NVARCHAR as NVARCHAR,
    REAL as REAL,
    SMALLINT as SMALLINT,
    TEXT as TEXT,
    TIME as TIME,
    TIMESTAMP as TIMESTAMP,
    VARBINARY as VARBINARY,
    VARCHAR as VARCHAR,
    Unicode as Unicode,
)

RESERVED_WORDS: Any

class _SybaseUnitypeMixin:
    def result_processor(self, dialect, coltype): ...

class UNICHAR(_SybaseUnitypeMixin, sqltypes.Unicode):
    __visit_name__: str

class UNIVARCHAR(_SybaseUnitypeMixin, sqltypes.Unicode):
    __visit_name__: str

class UNITEXT(_SybaseUnitypeMixin, sqltypes.UnicodeText):
    __visit_name__: str

class TINYINT(sqltypes.Integer):
    __visit_name__: str

class BIT(sqltypes.TypeEngine):
    __visit_name__: str

class MONEY(sqltypes.TypeEngine):
    __visit_name__: str

class SMALLMONEY(sqltypes.TypeEngine):
    __visit_name__: str

class UNIQUEIDENTIFIER(sqltypes.TypeEngine):
    __visit_name__: str

class IMAGE(sqltypes.LargeBinary):
    __visit_name__: str

class SybaseTypeCompiler(compiler.GenericTypeCompiler):
    def visit_large_binary(self, type_, **kw): ...
    def visit_boolean(self, type_, **kw): ...
    def visit_unicode(self, type_, **kw): ...
    def visit_UNICHAR(self, type_, **kw): ...
    def visit_UNIVARCHAR(self, type_, **kw): ...
    def visit_UNITEXT(self, type_, **kw): ...
    def visit_TINYINT(self, type_, **kw): ...
    def visit_IMAGE(self, type_, **kw): ...
    def visit_BIT(self, type_, **kw): ...
    def visit_MONEY(self, type_, **kw): ...
    def visit_SMALLMONEY(self, type_, **kw): ...
    def visit_UNIQUEIDENTIFIER(self, type_, **kw): ...

ischema_names: Any

class SybaseInspector(reflection.Inspector):
    def __init__(self, conn) -> None: ...
    def get_table_id(self, table_name, schema: Any | None = ...): ...

class SybaseExecutionContext(default.DefaultExecutionContext):
    def set_ddl_autocommit(self, connection, value) -> None: ...
    def pre_exec(self) -> None: ...
    def post_exec(self) -> None: ...
    def get_lastrowid(self): ...

class SybaseSQLCompiler(compiler.SQLCompiler):
    ansi_bind_rules: bool
    extract_map: Any
    def get_from_hint_text(self, table, text): ...
    def limit_clause(self, select, **kw): ...
    def visit_extract(self, extract, **kw): ...
    def visit_now_func(self, fn, **kw): ...
    def for_update_clause(self, select): ...
    def order_by_clause(self, select, **kw): ...
    def delete_table_clause(self, delete_stmt, from_table, extra_froms): ...
    def delete_extra_from_clause(self, delete_stmt, from_table, extra_froms, from_hints, **kw): ...

class SybaseDDLCompiler(compiler.DDLCompiler):
    def get_column_specification(self, column, **kwargs): ...
    def visit_drop_index(self, drop): ...

class SybaseIdentifierPreparer(compiler.IdentifierPreparer):
    reserved_words: Any

class SybaseDialect(default.DefaultDialect):
    name: str
    supports_unicode_statements: bool
    supports_sane_rowcount: bool
    supports_sane_multi_rowcount: bool
    supports_statement_cache: bool
    supports_native_boolean: bool
    supports_unicode_binds: bool
    postfetch_lastrowid: bool
    colspecs: Any
    ischema_names: Any
    type_compiler: Any
    statement_compiler: Any
    ddl_compiler: Any
    preparer: Any
    inspector: Any
    construct_arguments: Any
    def __init__(self, *args, **kwargs) -> None: ...
    max_identifier_length: int
    def initialize(self, connection) -> None: ...
    def get_table_id(self, connection, table_name, schema: Any | None = ..., **kw): ...
    def get_columns(self, connection, table_name, schema: Any | None = ..., **kw): ...
    def get_foreign_keys(self, connection, table_name, schema: Any | None = ..., **kw): ...
    def get_indexes(self, connection, table_name, schema: Any | None = ..., **kw): ...
    def get_pk_constraint(self, connection, table_name, schema: Any | None = ..., **kw): ...
    def get_schema_names(self, connection, **kw): ...
    def get_table_names(self, connection, schema: Any | None = ..., **kw): ...
    def get_view_definition(self, connection, view_name, schema: Any | None = ..., **kw): ...
    def get_view_names(self, connection, schema: Any | None = ..., **kw): ...
    def has_table(self, connection, table_name, schema: Any | None = ...): ...  # type: ignore[override]



from typing import Any

from . import base

class MockConnection(base.Connectable):
    def __init__(self, dialect, execute) -> None: ...
    engine: Any
    dialect: Any
    name: Any
    def schema_for_object(self, obj): ...
    def connect(self, **kwargs): ...
    def execution_options(self, **kw): ...
    def compiler(self, statement, parameters, **kwargs): ...
    def create(self, entity, **kwargs) -> None: ...
    def drop(self, entity, **kwargs) -> None: ...
    def execute(self, object_, *multiparams, **params) -> None: ...

def create_mock_engine(url, executor, **kw): ...



from typing import Any

from ..ext.hybrid import hybrid_property

class index_property(hybrid_property):
    attr_name: Any
    index: Any
    default: Any
    datatype: Any
    onebased: Any
    def __init__(self, attr_name, index, default=..., datatype: Any | None = ..., mutable: bool = ..., onebased: bool = ...): ...
    def fget(self, instance): ...
    def fset(self, instance, value) -> None: ...
    def fdel(self, instance) -> None: ...
    def expr(self, model): ...



from typing import Any

from ..engine import Connection as _LegacyConnection, Engine as _LegacyEngine
from ..engine.base import OptionEngineMixin

NO_OPTIONS: Any

def create_engine(*arg, **kw): ...

class Connection(_LegacyConnection):
    def begin(self): ...
    def begin_nested(self): ...
    def commit(self) -> None: ...
    def rollback(self) -> None: ...
    def close(self) -> None: ...
    def execute(self, statement, parameters: Any | None = ..., execution_options: Any | None = ...): ...  # type: ignore[override]
    def scalar(self, statement, parameters: Any | None = ..., execution_options: Any | None = ...): ...  # type: ignore[override]

class Engine(_LegacyEngine):
    transaction: Any
    run_callable: Any
    execute: Any
    scalar: Any
    table_names: Any
    has_table: Any
    def begin(self) -> None: ...  # type: ignore[override]
    def connect(self): ...

class OptionEngine(OptionEngineMixin, Engine): ...



from typing import Any

log: Any

class Bakery:
    cls: Any
    cache: Any
    def __init__(self, cls_, cache) -> None: ...
    def __call__(self, initial_fn, *args): ...

class BakedQuery:
    steps: Any
    def __init__(self, bakery, initial_fn, args=...) -> None: ...
    @classmethod
    def bakery(cls, size: int = ..., _size_alert: Any | None = ...): ...
    def __iadd__(self, other): ...
    def __add__(self, other): ...
    def add_criteria(self, fn, *args): ...
    def with_criteria(self, fn, *args): ...
    def for_session(self, session): ...
    def __call__(self, session): ...
    def spoil(self, full: bool = ...): ...
    def to_query(self, query_or_session): ...

class Result:
    bq: Any
    session: Any
    def __init__(self, bq, session) -> None: ...
    def params(self, *args, **kw): ...
    def with_post_criteria(self, fn): ...
    def __iter__(self): ...
    def count(self): ...
    def scalar(self): ...
    def first(self): ...
    def one(self): ...
    def one_or_none(self): ...
    def all(self): ...
    def get(self, ident): ...

def bake_lazy_loaders() -> None: ...
def unbake_lazy_loaders() -> None: ...

baked_lazyload: Any
baked_lazyload_all: Any
bakery: Any



from typing import Any

def expect_warnings(*messages, **kw): ...
def expect_warnings_on(db, *messages, **kw) -> None: ...
def emits_warning(*messages): ...
def expect_deprecated(*messages, **kw): ...
def expect_deprecated_20(*messages, **kw): ...
def emits_warning_on(db, *messages): ...
def uses_deprecated(*messages): ...
def global_cleanup_assertions() -> None: ...
def eq_regex(a, b, msg: Any | None = ...) -> None: ...
def eq_(a, b, msg: Any | None = ...) -> None: ...
def ne_(a, b, msg: Any | None = ...) -> None: ...
def le_(a, b, msg: Any | None = ...) -> None: ...
def is_instance_of(a, b, msg: Any | None = ...) -> None: ...
def is_none(a, msg: Any | None = ...) -> None: ...
def is_not_none(a, msg: Any | None = ...) -> None: ...
def is_true(a, msg: Any | None = ...) -> None: ...
def is_false(a, msg: Any | None = ...) -> None: ...
def is_(a, b, msg: Any | None = ...) -> None: ...
def is_not(a, b, msg: Any | None = ...) -> None: ...

is_not_ = is_not

def in_(a, b, msg: Any | None = ...) -> None: ...
def not_in(a, b, msg: Any | None = ...) -> None: ...

not_in_ = not_in

def startswith_(a, fragment, msg: Any | None = ...) -> None: ...
def eq_ignore_whitespace(a, b, msg: Any | None = ...) -> None: ...
def assert_raises(except_cls, callable_, *args, **kw): ...
def assert_raises_context_ok(except_cls, callable_, *args, **kw): ...
def assert_raises_message(except_cls, msg, callable_, *args, **kwargs): ...
def assert_raises_message_context_ok(except_cls, msg, callable_, *args, **kwargs): ...

class _ErrorContainer:
    error: Any

def expect_raises(except_cls, check_context: bool = ...): ...
def expect_raises_message(except_cls, msg, check_context: bool = ...): ...

class AssertsCompiledSQL:
    test_statement: Any
    supports_execution: Any
    def assert_compile(
        self,
        clause,
        result,
        params: Any | None = ...,
        checkparams: Any | None = ...,
        for_executemany: bool = ...,
        check_literal_execute: Any | None = ...,
        check_post_param: Any | None = ...,
        dialect: Any | None = ...,
        checkpositional: Any | None = ...,
        check_prefetch: Any | None = ...,
        use_default_dialect: bool = ...,
        allow_dialect_select: bool = ...,
        supports_default_values: bool = ...,
        supports_default_metavalue: bool = ...,
        literal_binds: bool = ...,
        render_postcompile: bool = ...,
        schema_translate_map: Any | None = ...,
        render_schema_translate: bool = ...,
        default_schema_name: Any | None = ...,
        from_linting: bool = ...,
    ): ...

class ComparesTables:
    def assert_tables_equal(self, table, reflected_table, strict_types: bool = ...) -> None: ...
    def assert_types_base(self, c1, c2) -> None: ...

class AssertsExecutionResults:
    def assert_result(self, result, class_, *objects) -> None: ...
    def assert_list(self, result, class_, list_) -> None: ...
    def assert_row(self, class_, rowobj, desc) -> None: ...
    def assert_unordered_result(self, result, cls, *expected): ...
    def sql_execution_asserter(self, db: Any | None = ...): ...
    def assert_sql_execution(self, db, callable_, *rules): ...
    def assert_sql(self, db, callable_, rules): ...
    def assert_sql_count(self, db, callable_, count) -> None: ...
    def assert_multiple_sql_count(self, dbs, callable_, counts): ...
    def assert_execution(self, db, *rules) -> None: ...
    def assert_statement_count(self, db, count): ...



from typing import Any

from .. import util
from .descriptor_props import (
    CompositeProperty as CompositeProperty,
    ConcreteInheritedProperty as ConcreteInheritedProperty,
    SynonymProperty as SynonymProperty,
)
from .interfaces import PropComparator, StrategizedProperty
from .relationships import RelationshipProperty as RelationshipProperty

__all__ = ["ColumnProperty", "CompositeProperty", "ConcreteInheritedProperty", "RelationshipProperty", "SynonymProperty"]

class ColumnProperty(StrategizedProperty):
    logger: Any
    strategy_wildcard_key: str
    inherit_cache: bool
    columns: Any
    group: Any
    deferred: Any
    raiseload: Any
    instrument: Any
    comparator_factory: Any
    descriptor: Any
    active_history: Any
    expire_on_flush: Any
    info: Any
    doc: Any
    strategy_key: Any
    def __init__(self, *columns, **kwargs) -> None: ...
    def __clause_element__(self): ...
    @property
    def expression(self): ...
    def instrument_class(self, mapper) -> None: ...
    def do_init(self) -> None: ...
    def copy(self): ...
    def merge(
        self, session, source_state, source_dict, dest_state, dest_dict, load, _recursive, _resolve_conflict_map
    ) -> None: ...
    class Comparator(util.MemoizedSlots, PropComparator):
        expressions: Any
        def _memoized_method___clause_element__(self): ...
        def operate(self, op, *other, **kwargs): ...
        def reverse_operate(self, op, other, **kwargs): ...



from ...exc import InvalidRequestError

class AsyncMethodRequired(InvalidRequestError): ...
class AsyncContextNotStarted(InvalidRequestError): ...
class AsyncContextAlreadyStarted(InvalidRequestError): ...



from typing import Any

def Serializer(*args, **kw): ...
def Deserializer(file, metadata: Any | None = ..., scoped_session: Any | None = ..., engine: Any | None = ...): ...
def dumps(obj, protocol=...): ...
def loads(data, metadata: Any | None = ..., scoped_session: Any | None = ..., engine: Any | None = ...): ...



import abc
from typing import Any

from ..util import memoized_property
from .result import Result, ResultMetaData
from .row import LegacyRow

MD_INDEX: int
MD_RESULT_MAP_INDEX: int
MD_OBJECTS: int
MD_LOOKUP_KEY: int
MD_RENDERED_NAME: int
MD_PROCESSOR: int
MD_UNTRANSLATED: int

class CursorResultMetaData(ResultMetaData):
    returns_rows: bool
    case_sensitive: Any
    def __init__(self, parent, cursor_description) -> None: ...

class LegacyCursorResultMetaData(CursorResultMetaData): ...

class ResultFetchStrategy:
    alternate_cursor_description: Any
    def soft_close(self, result, dbapi_cursor) -> None: ...
    def hard_close(self, result, dbapi_cursor) -> None: ...
    def yield_per(self, result, dbapi_cursor, num) -> None: ...
    def fetchone(self, result, dbapi_cursor, hard_close: bool = ...) -> None: ...
    def fetchmany(self, result, dbapi_cursor, size: Any | None = ...) -> None: ...
    def fetchall(self, result) -> None: ...
    def handle_exception(self, result, dbapi_cursor, err) -> None: ...

class NoCursorFetchStrategy(ResultFetchStrategy):
    def soft_close(self, result, dbapi_cursor) -> None: ...
    def hard_close(self, result, dbapi_cursor) -> None: ...
    def fetchone(self, result, dbapi_cursor, hard_close: bool = ...): ...
    def fetchmany(self, result, dbapi_cursor, size: Any | None = ...): ...
    def fetchall(self, result, dbapi_cursor): ...

class NoCursorDQLFetchStrategy(NoCursorFetchStrategy): ...
class NoCursorDMLFetchStrategy(NoCursorFetchStrategy): ...

class CursorFetchStrategy(ResultFetchStrategy):
    def soft_close(self, result, dbapi_cursor) -> None: ...
    def hard_close(self, result, dbapi_cursor) -> None: ...
    def handle_exception(self, result, dbapi_cursor, err) -> None: ...
    def yield_per(self, result, dbapi_cursor, num) -> None: ...
    def fetchone(self, result, dbapi_cursor, hard_close: bool = ...): ...
    def fetchmany(self, result, dbapi_cursor, size: Any | None = ...): ...
    def fetchall(self, result, dbapi_cursor): ...

class BufferedRowCursorFetchStrategy(CursorFetchStrategy):
    def __init__(self, dbapi_cursor, execution_options, growth_factor: int = ..., initial_buffer: Any | None = ...) -> None: ...
    @classmethod
    def create(cls, result): ...
    def yield_per(self, result, dbapi_cursor, num) -> None: ...
    def soft_close(self, result, dbapi_cursor) -> None: ...
    def hard_close(self, result, dbapi_cursor) -> None: ...
    def fetchone(self, result, dbapi_cursor, hard_close: bool = ...): ...
    def fetchmany(self, result, dbapi_cursor, size: Any | None = ...): ...
    def fetchall(self, result, dbapi_cursor): ...

class FullyBufferedCursorFetchStrategy(CursorFetchStrategy):
    alternate_cursor_description: Any
    def __init__(self, dbapi_cursor, alternate_description: Any | None = ..., initial_buffer: Any | None = ...) -> None: ...
    def yield_per(self, result, dbapi_cursor, num) -> None: ...
    def soft_close(self, result, dbapi_cursor) -> None: ...
    def hard_close(self, result, dbapi_cursor) -> None: ...
    def fetchone(self, result, dbapi_cursor, hard_close: bool = ...): ...
    def fetchmany(self, result, dbapi_cursor, size: Any | None = ...): ...
    def fetchall(self, result, dbapi_cursor): ...

class _NoResultMetaData(ResultMetaData):
    returns_rows: bool
    @property
    def keys(self) -> None: ...

class _LegacyNoResultMetaData(_NoResultMetaData):
    @property
    def keys(self): ...

class BaseCursorResult:
    out_parameters: Any
    closed: bool
    context: Any
    dialect: Any
    cursor: Any
    cursor_strategy: Any
    connection: Any
    def __init__(self, context, cursor_strategy, cursor_description): ...
    @property
    def inserted_primary_key_rows(self): ...
    @property
    def inserted_primary_key(self): ...
    def last_updated_params(self): ...
    def last_inserted_params(self): ...
    @property
    def returned_defaults_rows(self): ...
    @property
    def returned_defaults(self): ...
    def lastrow_has_defaults(self): ...
    def postfetch_cols(self): ...
    def prefetch_cols(self): ...
    def supports_sane_rowcount(self): ...
    def supports_sane_multi_rowcount(self): ...
    @memoized_property
    def rowcount(self): ...
    @property
    def lastrowid(self): ...
    @property
    def returns_rows(self): ...
    @property
    def is_insert(self): ...

class CursorResult(BaseCursorResult, Result):
    def merge(self, *others): ...
    def close(self) -> None: ...
    def yield_per(self, num) -> None: ...

class LegacyCursorResult(CursorResult):
    def close(self) -> None: ...

ResultProxy = LegacyCursorResult

class BufferedRowResultProxy(ResultProxy): ...
class FullyBufferedResultProxy(ResultProxy): ...
class BufferedColumnRow(LegacyRow, metaclass=abc.ABCMeta): ...
class BufferedColumnResultProxy(ResultProxy): ...



from typing import Any, NamedTuple

from .. import util
from .interfaces import LoaderStrategy

class UninstrumentedColumnLoader(LoaderStrategy):
    columns: Any
    def __init__(self, parent, strategy_key) -> None: ...
    def setup_query(
        self, compile_state, query_entity, path, loadopt, adapter, column_collection: Any | None = ..., **kwargs
    ) -> None: ...
    def create_row_processor(self, context, query_entity, path, loadopt, mapper, result, adapter, populators) -> None: ...

class ColumnLoader(LoaderStrategy):
    logger: Any
    columns: Any
    is_composite: Any
    def __init__(self, parent, strategy_key) -> None: ...
    def setup_query(self, compile_state, query_entity, path, loadopt, adapter, column_collection, memoized_populators, check_for_adapt: bool = ..., **kwargs) -> None: ...  # type: ignore[override]
    is_class_level: bool
    def init_class_attribute(self, mapper) -> None: ...
    def create_row_processor(self, context, query_entity, path, loadopt, mapper, result, adapter, populators) -> None: ...

class ExpressionColumnLoader(ColumnLoader):
    logger: Any
    def __init__(self, parent, strategy_key) -> None: ...
    def setup_query(self, compile_state, query_entity, path, loadopt, adapter, column_collection, memoized_populators, **kwargs) -> None: ...  # type: ignore[override]
    def create_row_processor(self, context, query_entity, path, loadopt, mapper, result, adapter, populators) -> None: ...
    is_class_level: bool
    def init_class_attribute(self, mapper) -> None: ...

class DeferredColumnLoader(LoaderStrategy):
    logger: Any
    raiseload: Any
    columns: Any
    group: Any
    def __init__(self, parent, strategy_key) -> None: ...
    def create_row_processor(self, context, query_entity, path, loadopt, mapper, result, adapter, populators) -> None: ...
    is_class_level: bool
    def init_class_attribute(self, mapper) -> None: ...
    def setup_query(self, compile_state, query_entity, path, loadopt, adapter, column_collection, memoized_populators, only_load_props: Any | None = ..., **kw) -> None: ...  # type: ignore[override]

class LoadDeferredColumns:
    key: Any
    raiseload: Any
    def __init__(self, key, raiseload: bool = ...) -> None: ...
    def __call__(self, state, passive=...): ...

class AbstractRelationshipLoader(LoaderStrategy):
    mapper: Any
    entity: Any
    target: Any
    uselist: Any
    def __init__(self, parent, strategy_key) -> None: ...

class DoNothingLoader(LoaderStrategy):
    logger: Any

class NoLoader(AbstractRelationshipLoader):
    logger: Any
    is_class_level: bool
    def init_class_attribute(self, mapper) -> None: ...
    def create_row_processor(self, context, query_entity, path, loadopt, mapper, result, adapter, populators) -> None: ...

class LazyLoader(AbstractRelationshipLoader, util.MemoizedSlots):
    logger: Any
    is_aliased_class: Any
    use_get: Any
    def __init__(self, parent, strategy_key) -> None: ...
    is_class_level: bool
    def init_class_attribute(self, mapper) -> None: ...
    def create_row_processor(self, context, query_entity, path, loadopt, mapper, result, adapter, populators) -> None: ...

class LoadLazyAttribute:
    key: Any
    strategy_key: Any
    loadopt: Any
    extra_criteria: Any
    def __init__(self, key, initiating_strategy, loadopt, extra_criteria) -> None: ...
    def __call__(self, state, passive=...): ...

class PostLoader(AbstractRelationshipLoader): ...

class ImmediateLoader(PostLoader):
    def init_class_attribute(self, mapper) -> None: ...
    def create_row_processor(self, context, query_entity, path, loadopt, mapper, result, adapter, populators) -> None: ...

class SubqueryLoader(PostLoader):
    logger: Any
    join_depth: Any
    def __init__(self, parent, strategy_key) -> None: ...
    def init_class_attribute(self, mapper) -> None: ...
    class _SubqCollections:
        session: Any
        execution_options: Any
        load_options: Any
        params: Any
        subq: Any
        def __init__(self, context, subq) -> None: ...
        def get(self, key, default): ...
        def loader(self, state, dict_, row) -> None: ...
    def create_row_processor(self, context, query_entity, path, loadopt, mapper, result, adapter, populators): ...

class JoinedLoader(AbstractRelationshipLoader):
    logger: Any
    join_depth: Any
    def __init__(self, parent, strategy_key) -> None: ...
    def init_class_attribute(self, mapper) -> None: ...
    def setup_query(
        self,
        compile_state,
        query_entity,
        path,
        loadopt,
        adapter,
        column_collection: Any | None = ...,
        parentmapper: Any | None = ...,
        chained_from_outerjoin: bool = ...,
        **kwargs,
    ) -> None: ...
    def create_row_processor(self, context, query_entity, path, loadopt, mapper, result, adapter, populators) -> None: ...

class SelectInLoader(PostLoader, util.MemoizedSlots):
    logger: Any
    class query_info(NamedTuple):
        load_only_child: Any
        load_with_join: Any
        in_expr: Any
        pk_cols: Any
        zero_idx: Any
        child_lookup_cols: Any
    join_depth: Any
    omit_join: Any
    def __init__(self, parent, strategy_key) -> None: ...
    def init_class_attribute(self, mapper) -> None: ...
    def create_row_processor(self, context, query_entity, path, loadopt, mapper, result, adapter, populators): ...

def single_parent_validator(desc, prop): ...



from typing import Any

from . import elements, roles
from .base import Options
from .operators import ColumnOperators

class LambdaOptions(Options):
    enable_tracking: bool
    track_closure_variables: bool
    track_on: Any
    global_track_bound_values: bool
    track_bound_values: bool
    lambda_cache: Any

def lambda_stmt(
    lmb,
    enable_tracking: bool = ...,
    track_closure_variables: bool = ...,
    track_on: Any | None = ...,
    global_track_bound_values: bool = ...,
    track_bound_values: bool = ...,
    lambda_cache: Any | None = ...,
): ...

class LambdaElement(elements.ClauseElement):
    __visit_name__: str
    parent_lambda: Any
    fn: Any
    role: Any
    tracker_key: Any
    opts: Any
    def __init__(self, fn, role, opts=..., apply_propagate_attrs: Any | None = ...) -> None: ...
    def __getattr__(self, key): ...

class DeferredLambdaElement(LambdaElement):
    lambda_args: Any
    def __init__(self, fn, role, opts=..., lambda_args=...) -> None: ...

class StatementLambdaElement(roles.AllowsLambdaRole, LambdaElement):
    def __add__(self, other): ...
    def add_criteria(
        self,
        other,
        enable_tracking: bool = ...,
        track_on: Any | None = ...,
        track_closure_variables: bool = ...,
        track_bound_values: bool = ...,
    ): ...
    def spoil(self): ...

class NullLambdaStatement(roles.AllowsLambdaRole, elements.ClauseElement):
    __visit_name__: str
    def __init__(self, statement) -> None: ...
    def __getattr__(self, key): ...
    def __add__(self, other): ...
    def add_criteria(self, other, **kw): ...

class LinkedLambdaElement(StatementLambdaElement):
    role: Any
    opts: Any
    fn: Any
    parent_lambda: Any
    tracker_key: Any
    def __init__(self, fn, parent_lambda, opts) -> None: ...

class AnalyzedCode:
    @classmethod
    def get(cls, fn, lambda_element, lambda_kw, **kw): ...
    track_bound_values: Any
    track_closure_variables: Any
    bindparam_trackers: Any
    closure_trackers: Any
    build_py_wrappers: Any
    def __init__(self, fn, lambda_element, opts) -> None: ...

class NonAnalyzedFunction:
    closure_bindparams: Any
    bindparam_trackers: Any
    expr: Any
    def __init__(self, expr) -> None: ...
    @property
    def expected_expr(self): ...

class AnalyzedFunction:
    analyzed_code: Any
    fn: Any
    closure_pywrappers: Any
    tracker_instrumented_fn: Any
    expr: Any
    bindparam_trackers: Any
    expected_expr: Any
    is_sequence: Any
    propagate_attrs: Any
    closure_bindparams: Any
    def __init__(self, analyzed_code, lambda_element, apply_propagate_attrs, fn) -> None: ...

class PyWrapper(ColumnOperators):
    fn: Any
    track_bound_values: Any
    def __init__(
        self, fn, name, to_evaluate, closure_index: Any | None = ..., getter: Any | None = ..., track_bound_values: bool = ...
    ) -> None: ...
    def __call__(self, *arg, **kw): ...
    def operate(self, op, *other, **kwargs): ...
    def reverse_operate(self, op, other, **kwargs): ...
    def __clause_element__(self): ...
    def __bool__(self): ...
    def __nonzero__(self): ...
    def __getattribute__(self, key): ...
    def __iter__(self): ...
    def __getitem__(self, key): ...

def insp(lmb): ...



from typing import Any

from .. import log
from .interfaces import Connectable as Connectable, ExceptionContext
from .util import TransactionalContext

class Connection(Connectable):
    engine: Any
    dialect: Any
    should_close_with_result: bool
    dispatch: Any
    def __init__(
        self,
        engine,
        connection: Any | None = ...,
        close_with_result: bool = ...,
        _branch_from: Any | None = ...,
        _execution_options: Any | None = ...,
        _dispatch: Any | None = ...,
        _has_events: Any | None = ...,
        _allow_revalidate: bool = ...,
    ) -> None: ...
    def schema_for_object(self, obj): ...
    def __enter__(self): ...
    def __exit__(self, type_, value, traceback) -> None: ...
    def execution_options(self, **opt): ...
    def get_execution_options(self): ...
    @property
    def closed(self): ...
    @property
    def invalidated(self): ...
    @property
    def connection(self): ...
    def get_isolation_level(self): ...
    @property
    def default_isolation_level(self): ...
    @property
    def info(self): ...
    def connect(self, close_with_result: bool = ...): ...  # type: ignore[override]
    def invalidate(self, exception: Any | None = ...): ...
    def detach(self) -> None: ...
    def begin(self): ...
    def begin_nested(self): ...
    def begin_twophase(self, xid: Any | None = ...): ...
    def recover_twophase(self): ...
    def rollback_prepared(self, xid, recover: bool = ...) -> None: ...
    def commit_prepared(self, xid, recover: bool = ...) -> None: ...
    def in_transaction(self): ...
    def in_nested_transaction(self): ...
    def get_transaction(self): ...
    def get_nested_transaction(self): ...
    def close(self) -> None: ...
    def scalar(self, object_, *multiparams, **params): ...
    def scalars(self, object_, *multiparams, **params): ...
    def execute(self, statement, *multiparams, **params): ...
    def exec_driver_sql(self, statement, parameters: Any | None = ..., execution_options: Any | None = ...): ...
    def transaction(self, callable_, *args, **kwargs): ...
    def run_callable(self, callable_, *args, **kwargs): ...

class ExceptionContextImpl(ExceptionContext):
    engine: Any
    connection: Any
    sqlalchemy_exception: Any
    original_exception: Any
    execution_context: Any
    statement: Any
    parameters: Any
    is_disconnect: Any
    invalidate_pool_on_disconnect: Any
    def __init__(
        self,
        exception,
        sqlalchemy_exception,
        engine,
        connection,
        cursor,
        statement,
        parameters,
        context,
        is_disconnect,
        invalidate_pool_on_disconnect,
    ) -> None: ...

class Transaction(TransactionalContext):
    def __init__(self, connection) -> None: ...
    @property
    def is_valid(self): ...
    def close(self) -> None: ...
    def rollback(self) -> None: ...
    def commit(self) -> None: ...

class MarkerTransaction(Transaction):
    connection: Any
    def __init__(self, connection) -> None: ...
    @property
    def is_active(self): ...

class RootTransaction(Transaction):
    connection: Any
    is_active: bool
    def __init__(self, connection) -> None: ...

class NestedTransaction(Transaction):
    connection: Any
    is_active: bool
    def __init__(self, connection) -> None: ...

class TwoPhaseTransaction(RootTransaction):
    xid: Any
    def __init__(self, connection, xid) -> None: ...
    def prepare(self) -> None: ...

class Engine(Connectable, log.Identified):
    pool: Any
    url: Any
    dialect: Any
    logging_name: Any
    echo: Any
    hide_parameters: Any
    def __init__(
        self,
        pool,
        dialect,
        url,
        logging_name: Any | None = ...,
        echo: Any | None = ...,
        query_cache_size: int = ...,
        execution_options: Any | None = ...,
        hide_parameters: bool = ...,
    ) -> None: ...
    @property
    def engine(self): ...
    def clear_compiled_cache(self) -> None: ...
    def update_execution_options(self, **opt) -> None: ...
    def execution_options(self, **opt): ...
    def get_execution_options(self): ...
    @property
    def name(self): ...
    @property
    def driver(self): ...
    def dispose(self) -> None: ...
    class _trans_ctx:
        conn: Any
        transaction: Any
        close_with_result: Any
        def __init__(self, conn, transaction, close_with_result) -> None: ...
        def __enter__(self): ...
        def __exit__(self, type_, value, traceback) -> None: ...
    def begin(self, close_with_result: bool = ...): ...
    def transaction(self, callable_, *args, **kwargs): ...
    def run_callable(self, callable_, *args, **kwargs): ...
    def execute(self, statement, *multiparams, **params): ...
    def scalar(self, statement, *multiparams, **params): ...
    def connect(self, close_with_result: bool = ...): ...  # type: ignore[override]
    def table_names(self, schema: Any | None = ..., connection: Any | None = ...): ...
    def has_table(self, table_name, schema: Any | None = ...): ...
    def raw_connection(self, _connection: Any | None = ...): ...

class OptionEngineMixin:
    url: Any
    dialect: Any
    logging_name: Any
    echo: Any
    hide_parameters: Any
    dispatch: Any
    def __init__(self, proxied, execution_options) -> None: ...
    pool: Any

class OptionEngine(OptionEngineMixin, Engine): ...



from ...engine import events as engine_event
from ...orm import events as orm_event

class AsyncConnectionEvents(engine_event.ConnectionEvents): ...
class AsyncSessionEvents(orm_event.SessionEvents): ...



def add_fixture(fn, fixture) -> None: ...
def scan_for_fixtures_to_use_for_class(item) -> None: ...
def run_class_fixture_setup(request) -> None: ...
def run_class_fixture_teardown(request) -> None: ...
def run_fn_fixture_setup(request) -> None: ...
def run_fn_fixture_teardown(request) -> None: ...



from typing import Any

from . import Connector

class PyODBCConnector(Connector):
    driver: str
    supports_sane_rowcount_returning: bool
    supports_sane_multi_rowcount: bool
    supports_unicode_statements: bool
    supports_unicode_binds: bool
    supports_native_decimal: bool
    default_paramstyle: str
    use_setinputsizes: bool
    pyodbc_driver_name: Any
    def __init__(self, supports_unicode_binds: Any | None = ..., use_setinputsizes: bool = ..., **kw) -> None: ...
    @classmethod
    def dbapi(cls): ...
    def create_connect_args(self, url): ...
    def is_disconnect(self, e, connection, cursor): ...
    def do_set_input_sizes(self, cursor, list_of_tuples, context) -> None: ...
    def set_isolation_level(self, connection, level) -> None: ...



from typing import Any

from ...util import memoized_property
from .base import MySQLCompiler, MySQLDialect, MySQLExecutionContext

class MySQLExecutionContext_mysqldb(MySQLExecutionContext):
    @property
    def rowcount(self): ...

class MySQLCompiler_mysqldb(MySQLCompiler): ...

class MySQLDialect_mysqldb(MySQLDialect):
    driver: str
    supports_statement_cache: bool
    supports_unicode_statements: bool
    supports_sane_rowcount: bool
    supports_sane_multi_rowcount: bool
    supports_native_decimal: bool
    default_paramstyle: str
    statement_compiler: Any
    preparer: Any
    def __init__(self, **kwargs) -> None: ...
    @memoized_property
    def supports_server_side_cursors(self): ...
    @classmethod
    def dbapi(cls): ...
    def on_connect(self): ...
    def do_ping(self, dbapi_connection): ...
    def do_executemany(self, cursor, statement, parameters, context: Any | None = ...) -> None: ...
    def create_connect_args(self, url, _translate_args: Any | None = ...): ...

dialect = MySQLDialect_mysqldb



from _typeshed import Self
from logging import Logger
from typing import Any, TypeVar, overload
from typing_extensions import Literal

_ClsT = TypeVar("_ClsT", bound=type)
_EchoFlag = bool | Literal["debug"] | None

rootlogger: Any

def class_logger(cls: _ClsT) -> _ClsT: ...

class Identified:
    logging_name: str | None

class InstanceLogger:
    echo: _EchoFlag
    logger: Logger
    def __init__(self, echo: _EchoFlag, name: str | None) -> None: ...
    def debug(self, msg, *args, **kwargs) -> None: ...
    def info(self, msg, *args, **kwargs) -> None: ...
    def warning(self, msg, *args, **kwargs) -> None: ...
    warn = warning
    def error(self, msg, *args, **kwargs) -> None: ...
    def exception(self, msg, *args, **kwargs) -> None: ...
    def critical(self, msg, *args, **kwargs) -> None: ...
    def log(self, level, msg, *args, **kwargs) -> None: ...
    def isEnabledFor(self, level): ...
    def getEffectiveLevel(self): ...

def instance_logger(instance: Identified, echoflag: _EchoFlag = ...) -> None: ...

class echo_property:
    __doc__: str
    @overload
    def __get__(self: Self, instance: None, owner: object) -> Self: ...
    @overload
    def __get__(self, instance: Identified, owner: object) -> _EchoFlag: ...
    def __set__(self, instance: Identified, value: _EchoFlag) -> None: ...



from typing import Any

from ..util import memoized_property

PASSIVE_NO_RESULT: Any
PASSIVE_CLASS_MISMATCH: Any
ATTR_WAS_SET: Any
ATTR_EMPTY: Any
NO_VALUE: Any
NEVER_SET: Any
NO_CHANGE: Any
CALLABLES_OK: Any
SQL_OK: Any
RELATED_OBJECT_OK: Any
INIT_OK: Any
NON_PERSISTENT_OK: Any
LOAD_AGAINST_COMMITTED: Any
NO_AUTOFLUSH: Any
NO_RAISE: Any
DEFERRED_HISTORY_LOAD: Any
PASSIVE_OFF: Any
PASSIVE_RETURN_NO_VALUE: Any
PASSIVE_NO_INITIALIZE: Any
PASSIVE_NO_FETCH: Any
PASSIVE_NO_FETCH_RELATED: Any
PASSIVE_ONLY_PERSISTENT: Any
DEFAULT_MANAGER_ATTR: str
DEFAULT_STATE_ATTR: str
EXT_CONTINUE: Any
EXT_STOP: Any
EXT_SKIP: Any
ONETOMANY: Any
MANYTOONE: Any
MANYTOMANY: Any
NOT_EXTENSION: Any

_never_set: frozenset[Any]
_none_set: frozenset[Any]

def manager_of_class(cls): ...

instance_state: Any
instance_dict: Any

def instance_str(instance): ...
def state_str(state): ...
def state_class_str(state): ...
def attribute_str(instance, attribute): ...
def state_attribute_str(state, attribute): ...
def object_mapper(instance): ...
def object_state(instance): ...
def _class_to_mapper(class_or_mapper): ...
def _mapper_or_none(entity): ...
def _is_mapped_class(entity): ...

_state_mapper: Any

def class_mapper(class_, configure: bool = ...): ...

class InspectionAttr:
    is_selectable: bool
    is_aliased_class: bool
    is_instance: bool
    is_mapper: bool
    is_bundle: bool
    is_property: bool
    is_attribute: bool
    is_clause_element: bool
    extension_type: Any

class InspectionAttrInfo(InspectionAttr):
    @memoized_property
    def info(self): ...

class _MappedAttribute: ...



from typing import Any

from ...util import memoized_property
from .mysqldb import MySQLDialect_mysqldb

class MySQLDialect_pymysql(MySQLDialect_mysqldb):
    driver: str
    supports_statement_cache: bool
    description_encoding: Any
    supports_unicode_statements: bool
    supports_unicode_binds: bool
    @memoized_property
    def supports_server_side_cursors(self): ...
    @classmethod
    def dbapi(cls): ...
    def create_connect_args(self, url, _translate_args: Any | None = ...): ...
    def is_disconnect(self, e, connection, cursor): ...

dialect = MySQLDialect_pymysql



from typing import Any, ClassVar, Type

from .. import types as sqltypes
from ..util import memoized_property
from . import interfaces

AUTOCOMMIT_REGEXP: Any
SERVER_SIDE_CURSOR_RE: Any
CACHE_HIT: Any
CACHE_MISS: Any
CACHING_DISABLED: Any
NO_CACHE_KEY: Any
NO_DIALECT_SUPPORT: Any

class DefaultDialect(interfaces.Dialect):
    execution_ctx_cls: ClassVar[Type[interfaces.ExecutionContext]]
    statement_compiler: Any
    ddl_compiler: Any
    type_compiler: Any
    preparer: Any
    supports_alter: bool
    supports_comments: bool
    inline_comments: bool
    use_setinputsizes: bool
    supports_statement_cache: bool
    default_sequence_base: int
    execute_sequence_format: Any
    supports_schemas: bool
    supports_views: bool
    supports_sequences: bool
    sequences_optional: bool
    preexecute_autoincrement_sequences: bool
    supports_identity_columns: bool
    postfetch_lastrowid: bool
    implicit_returning: bool
    full_returning: bool
    insert_executemany_returning: bool
    cte_follows_insert: bool
    supports_native_enum: bool
    supports_native_boolean: bool
    non_native_boolean_check_constraint: bool
    supports_simple_order_by_label: bool
    tuple_in_values: bool
    connection_characteristics: Any
    engine_config_types: Any
    supports_native_decimal: bool
    supports_unicode_statements: bool
    supports_unicode_binds: bool
    returns_unicode_strings: Any
    description_encoding: Any
    name: str
    max_identifier_length: int
    isolation_level: Any
    max_index_name_length: Any
    max_constraint_name_length: Any
    supports_sane_rowcount: bool
    supports_sane_multi_rowcount: bool
    colspecs: Any
    default_paramstyle: str
    supports_default_values: bool
    supports_default_metavalue: bool
    supports_empty_insert: bool
    supports_multivalues_insert: bool
    supports_is_distinct_from: bool
    supports_server_side_cursors: bool
    server_side_cursors: bool
    supports_for_update_of: bool
    server_version_info: Any
    default_schema_name: Any
    construct_arguments: Any
    requires_name_normalize: bool
    reflection_options: Any
    dbapi_exception_translation_map: Any
    is_async: bool
    CACHE_HIT: Any
    CACHE_MISS: Any
    CACHING_DISABLED: Any
    NO_CACHE_KEY: Any
    NO_DIALECT_SUPPORT: Any
    convert_unicode: Any
    encoding: Any
    positional: bool
    dbapi: Any
    paramstyle: Any
    identifier_preparer: Any
    case_sensitive: Any
    label_length: Any
    compiler_linting: Any
    def __init__(
        self,
        convert_unicode: bool = ...,
        encoding: str = ...,
        paramstyle: Any | None = ...,
        dbapi: Any | None = ...,
        implicit_returning: Any | None = ...,
        case_sensitive: bool = ...,
        supports_native_boolean: Any | None = ...,
        max_identifier_length: Any | None = ...,
        label_length: Any | None = ...,
        compiler_linting=...,
        server_side_cursors: bool = ...,
        **kwargs,
    ) -> None: ...
    @property
    def dialect_description(self): ...
    @property
    def supports_sane_rowcount_returning(self): ...
    @classmethod
    def get_pool_class(cls, url): ...
    def get_dialect_pool_class(self, url): ...
    @classmethod
    def load_provisioning(cls) -> None: ...
    default_isolation_level: Any
    def initialize(self, connection) -> None: ...
    def on_connect(self) -> None: ...
    def get_default_isolation_level(self, dbapi_conn): ...
    def type_descriptor(self, typeobj): ...
    def has_index(self, connection, table_name, index_name, schema: Any | None = ...): ...
    def validate_identifier(self, ident) -> None: ...
    def connect(self, *cargs, **cparams): ...
    def create_connect_args(self, url): ...
    def set_engine_execution_options(self, engine, opts) -> None: ...
    def set_connection_execution_options(self, connection, opts) -> None: ...
    def do_begin(self, dbapi_connection) -> None: ...
    def do_rollback(self, dbapi_connection) -> None: ...
    def do_commit(self, dbapi_connection) -> None: ...
    def do_close(self, dbapi_connection) -> None: ...
    def do_ping(self, dbapi_connection): ...
    def create_xid(self): ...
    def do_savepoint(self, connection, name) -> None: ...
    def do_rollback_to_savepoint(self, connection, name) -> None: ...
    def do_release_savepoint(self, connection, name) -> None: ...
    def do_executemany(self, cursor, statement, parameters, context: Any | None = ...) -> None: ...
    def do_execute(self, cursor, statement, parameters, context: Any | None = ...) -> None: ...
    def do_execute_no_params(self, cursor, statement, context: Any | None = ...) -> None: ...  # type: ignore[override]
    def is_disconnect(self, e, connection, cursor): ...
    def reset_isolation_level(self, dbapi_conn) -> None: ...
    def normalize_name(self, name): ...
    def denormalize_name(self, name): ...
    def get_driver_connection(self, connection): ...

class _RendersLiteral:
    def literal_processor(self, dialect): ...

class _StrDateTime(_RendersLiteral, sqltypes.DateTime): ...
class _StrDate(_RendersLiteral, sqltypes.Date): ...
class _StrTime(_RendersLiteral, sqltypes.Time): ...

class StrCompileDialect(DefaultDialect):
    statement_compiler: Any
    ddl_compiler: Any
    type_compiler: Any
    preparer: Any
    supports_statement_cache: bool
    supports_identity_columns: bool
    supports_sequences: bool
    sequences_optional: bool
    preexecute_autoincrement_sequences: bool
    implicit_returning: bool
    supports_native_boolean: bool
    supports_multivalues_insert: bool
    supports_simple_order_by_label: bool
    colspecs: Any

class DefaultExecutionContext(interfaces.ExecutionContext):
    isinsert: bool
    isupdate: bool
    isdelete: bool
    is_crud: bool
    is_text: bool
    isddl: bool
    executemany: bool
    compiled: Any
    statement: Any
    result_column_struct: Any
    returned_default_rows: Any
    execution_options: Any
    include_set_input_sizes: Any
    exclude_set_input_sizes: Any
    cursor_fetch_strategy: Any
    cache_stats: Any
    invoked_statement: Any
    cache_hit: Any
    @memoized_property
    def identifier_preparer(self): ...
    @memoized_property
    def engine(self): ...
    @memoized_property
    def postfetch_cols(self): ...
    @memoized_property
    def prefetch_cols(self): ...
    @memoized_property
    def returning_cols(self) -> None: ...
    @memoized_property
    def no_parameters(self): ...
    @memoized_property
    def should_autocommit(self): ...
    @property
    def connection(self): ...
    def should_autocommit_text(self, statement): ...
    def create_cursor(self): ...
    def create_default_cursor(self): ...
    def create_server_side_cursor(self) -> None: ...
    def pre_exec(self) -> None: ...
    def get_out_parameter_values(self, names) -> None: ...
    def post_exec(self) -> None: ...
    def get_result_processor(self, type_, colname, coltype): ...
    def get_lastrowid(self): ...
    def handle_dbapi_exception(self, e) -> None: ...
    @property
    def rowcount(self): ...
    def supports_sane_rowcount(self): ...
    def supports_sane_multi_rowcount(self): ...
    @memoized_property
    def inserted_primary_key_rows(self): ...
    def lastrow_has_defaults(self): ...
    current_parameters: Any
    def get_current_parameters(self, isolate_multiinsert_groups: bool = ...): ...
    def get_insert_default(self, column): ...
    def get_update_default(self, column): ...



from typing import Any

from .. import exc as sa_exc

NO_STATE: Any

class StaleDataError(sa_exc.SQLAlchemyError): ...

ConcurrentModificationError = StaleDataError

class FlushError(sa_exc.SQLAlchemyError): ...
class UnmappedError(sa_exc.InvalidRequestError): ...
class ObjectDereferencedError(sa_exc.SQLAlchemyError): ...

class DetachedInstanceError(sa_exc.SQLAlchemyError):
    code: str

class UnmappedInstanceError(UnmappedError):
    def __init__(self, obj, msg: Any | None = ...) -> None: ...
    def __reduce__(self): ...

class UnmappedClassError(UnmappedError):
    def __init__(self, cls, msg: Any | None = ...) -> None: ...
    def __reduce__(self): ...

class ObjectDeletedError(sa_exc.InvalidRequestError):
    def __init__(self, state, msg: Any | None = ...) -> None: ...
    def __reduce__(self): ...

class UnmappedColumnError(sa_exc.InvalidRequestError): ...

class LoaderStrategyException(sa_exc.InvalidRequestError):
    def __init__(self, applied_to_property_type, requesting_property, applies_to, actual_strategy_type, strategy_key) -> None: ...



from typing import Any

from ...orm.scoping import ScopedSessionMixin
from ...util import memoized_property

class async_scoped_session(ScopedSessionMixin):
    session_factory: Any
    registry: Any
    def __init__(self, session_factory, scopefunc) -> None: ...
    async def remove(self) -> None: ...
    # proxied from Session
    @classmethod
    async def close_all(cls): ...
    @classmethod
    def identity_key(cls, *args, **kwargs): ...
    @classmethod
    def object_session(cls, instance): ...
    bind: Any
    identity_map: Any
    autoflush: Any
    def __contains__(self, instance): ...
    def __iter__(self): ...
    def add(self, instance, _warn: bool = ...) -> None: ...
    def add_all(self, instances) -> None: ...
    def begin(self, **kw): ...
    def begin_nested(self, **kw): ...
    async def close(self): ...
    async def commit(self): ...
    async def connection(self, **kw): ...
    async def delete(self, instance): ...
    async def execute(
        self, statement, params: Any | None = ..., execution_options=..., bind_arguments: Any | None = ..., **kw
    ): ...
    def expire(self, instance, attribute_names: Any | None = ...) -> None: ...
    def expire_all(self) -> None: ...
    def expunge(self, instance) -> None: ...
    def expunge_all(self) -> None: ...
    async def flush(self, objects: Any | None = ...) -> None: ...
    async def get(
        self,
        entity,
        ident,
        options: Any | None = ...,
        populate_existing: bool = ...,
        with_for_update: Any | None = ...,
        identity_token: Any | None = ...,
    ): ...
    def get_bind(self, mapper: Any | None = ..., clause: Any | None = ..., bind: Any | None = ..., **kw): ...
    def is_modified(self, instance, include_collections: bool = ...): ...
    async def merge(self, instance, load: bool = ..., options: Any | None = ...): ...
    async def refresh(self, instance, attribute_names: Any | None = ..., with_for_update: Any | None = ...): ...
    async def rollback(self): ...
    async def scalar(
        self, statement, params: Any | None = ..., execution_options=..., bind_arguments: Any | None = ..., **kw
    ): ...
    async def scalars(
        self, statement, params: Any | None = ..., execution_options=..., bind_arguments: Any | None = ..., **kw
    ): ...
    async def stream(
        self, statement, params: Any | None = ..., execution_options=..., bind_arguments: Any | None = ..., **kw
    ): ...
    async def stream_scalars(
        self, statement, params: Any | None = ..., execution_options=..., bind_arguments: Any | None = ..., **kw
    ): ...
    @property
    def dirty(self): ...
    @property
    def deleted(self): ...
    @property
    def new(self): ...
    @property
    def is_active(self): ...
    @property
    def no_autoflush(self) -> None: ...
    @memoized_property
    def info(self): ...



from typing import Any

from ..engine.util import TransactionalContext
from ..util import MemoizedSlots, memoized_property

class _SessionClassMethods:
    @classmethod
    def close_all(cls) -> None: ...
    @classmethod
    def identity_key(cls, *args, **kwargs): ...
    @classmethod
    def object_session(cls, instance): ...

class ORMExecuteState(MemoizedSlots):
    session: Any
    statement: Any
    parameters: Any
    local_execution_options: Any
    execution_options: Any
    bind_arguments: Any
    def __init__(
        self, session, statement, parameters, execution_options, bind_arguments, compile_state_cls, events_todo
    ) -> None: ...
    def invoke_statement(
        self,
        statement: Any | None = ...,
        params: Any | None = ...,
        execution_options: Any | None = ...,
        bind_arguments: Any | None = ...,
    ): ...
    @property
    def bind_mapper(self): ...
    @property
    def all_mappers(self): ...
    @property
    def is_orm_statement(self): ...
    @property
    def is_select(self): ...
    @property
    def is_insert(self): ...
    @property
    def is_update(self): ...
    @property
    def is_delete(self): ...
    def update_execution_options(self, **opts) -> None: ...
    @property
    def lazy_loaded_from(self): ...
    @property
    def loader_strategy_path(self): ...
    @property
    def is_column_load(self): ...
    @property
    def is_relationship_load(self): ...
    @property
    def load_options(self): ...
    @property
    def update_delete_options(self): ...
    @property
    def user_defined_options(self): ...

class SessionTransaction(TransactionalContext):
    session: Any
    nested: Any
    def __init__(self, session, parent: Any | None = ..., nested: bool = ..., autobegin: bool = ...) -> None: ...
    @property
    def parent(self): ...
    @property
    def is_active(self): ...
    def connection(self, bindkey, execution_options: Any | None = ..., **kwargs): ...
    def prepare(self) -> None: ...
    def commit(self, _to_root: bool = ...): ...
    def rollback(self, _capture_exception: bool = ..., _to_root: bool = ...): ...
    def close(self, invalidate: bool = ...) -> None: ...

class Session(_SessionClassMethods):
    identity_map: Any
    bind: Any
    future: Any
    hash_key: Any
    autoflush: Any
    expire_on_commit: Any
    enable_baked_queries: Any
    autocommit: bool
    twophase: Any
    def __init__(
        self,
        bind: Any | None = ...,
        autoflush: bool = ...,
        future: bool = ...,
        expire_on_commit: bool = ...,
        autocommit: bool = ...,
        twophase: bool = ...,
        binds: Any | None = ...,
        enable_baked_queries: bool = ...,
        info: Any | None = ...,
        query_cls: Any | None = ...,
    ) -> None: ...
    connection_callable: Any
    def __enter__(self): ...
    def __exit__(self, type_, value, traceback) -> None: ...
    @property
    def transaction(self): ...
    def in_transaction(self): ...
    def in_nested_transaction(self): ...
    def get_transaction(self): ...
    def get_nested_transaction(self): ...
    @memoized_property
    def info(self): ...
    def begin(self, subtransactions: bool = ..., nested: bool = ..., _subtrans: bool = ...): ...
    def begin_nested(self): ...
    def rollback(self) -> None: ...
    def commit(self) -> None: ...
    def prepare(self) -> None: ...
    def connection(
        self, bind_arguments: Any | None = ..., close_with_result: bool = ..., execution_options: Any | None = ..., **kw
    ): ...
    def execute(
        self,
        statement,
        params: Any | None = ...,
        execution_options=...,
        bind_arguments: Any | None = ...,
        _parent_execute_state: Any | None = ...,
        _add_event: Any | None = ...,
        **kw,
    ): ...
    def scalar(self, statement, params: Any | None = ..., execution_options=..., bind_arguments: Any | None = ..., **kw): ...
    def scalars(self, statement, params: Any | None = ..., execution_options=..., bind_arguments: Any | None = ..., **kw): ...
    def close(self) -> None: ...
    def invalidate(self) -> None: ...
    def expunge_all(self) -> None: ...
    def bind_mapper(self, mapper, bind) -> None: ...
    def bind_table(self, table, bind) -> None: ...
    def get_bind(
        self,
        mapper: Any | None = ...,
        clause: Any | None = ...,
        bind: Any | None = ...,
        _sa_skip_events: Any | None = ...,
        _sa_skip_for_implicit_returning: bool = ...,
    ): ...
    def query(self, *entities, **kwargs): ...
    @property
    def no_autoflush(self) -> None: ...
    def refresh(self, instance, attribute_names: Any | None = ..., with_for_update: Any | None = ...) -> None: ...
    def expire_all(self) -> None: ...
    def expire(self, instance, attribute_names: Any | None = ...) -> None: ...
    def expunge(self, instance) -> None: ...
    def add(self, instance, _warn: bool = ...) -> None: ...
    def add_all(self, instances) -> None: ...
    def delete(self, instance) -> None: ...
    def get(
        self,
        entity,
        ident,
        options: Any | None = ...,
        populate_existing: bool = ...,
        with_for_update: Any | None = ...,
        identity_token: Any | None = ...,
        execution_options: Any | None = ...,
    ): ...
    def merge(self, instance, load: bool = ..., options: Any | None = ...): ...
    def enable_relationship_loading(self, obj) -> None: ...
    def __contains__(self, instance): ...
    def __iter__(self): ...
    def flush(self, objects: Any | None = ...) -> None: ...
    def bulk_save_objects(
        self, objects, return_defaults: bool = ..., update_changed_only: bool = ..., preserve_order: bool = ...
    ): ...
    def bulk_insert_mappings(self, mapper, mappings, return_defaults: bool = ..., render_nulls: bool = ...) -> None: ...
    def bulk_update_mappings(self, mapper, mappings) -> None: ...
    def is_modified(self, instance, include_collections: bool = ...): ...
    @property
    def is_active(self): ...
    @property
    def dirty(self): ...
    @property
    def deleted(self): ...
    @property
    def new(self): ...

class sessionmaker(_SessionClassMethods):
    kw: Any
    class_: Any
    def __init__(
        self,
        bind: Any | None = ...,
        class_=...,
        autoflush: bool = ...,
        autocommit: bool = ...,
        expire_on_commit: bool = ...,
        info: Any | None = ...,
        **kw,
    ) -> None: ...
    def begin(self): ...
    def __call__(self, **local_kw): ...
    def configure(self, **new_kw) -> None: ...

def close_all_sessions() -> None: ...
def make_transient(instance) -> None: ...
def make_transient_to_detached(instance) -> None: ...
def object_session(instance): ...



from .base import Pool as Pool, reset_commit as reset_commit, reset_none as reset_none, reset_rollback as reset_rollback
from .dbapi_proxy import clear_managers as clear_managers, manage as manage
from .impl import (
    AssertionPool as AssertionPool,
    AsyncAdaptedQueuePool as AsyncAdaptedQueuePool,
    FallbackAsyncAdaptedQueuePool as FallbackAsyncAdaptedQueuePool,
    NullPool as NullPool,
    QueuePool as QueuePool,
    SingletonThreadPool as SingletonThreadPool,
    StaticPool as StaticPool,
)

__all__ = [
    "Pool",
    "reset_commit",
    "reset_none",
    "reset_rollback",
    "clear_managers",
    "manage",
    "AssertionPool",
    "NullPool",
    "QueuePool",
    "AsyncAdaptedQueuePool",
    "FallbackAsyncAdaptedQueuePool",
    "SingletonThreadPool",
    "StaticPool",
]



from typing import Any

from .base import PARSE_AUTOCOMMIT as PARSE_AUTOCOMMIT, ColumnCollection as ColumnCollection, Executable as Executable
from .dml import Delete as Delete, Insert as Insert, Update as Update, UpdateBase as UpdateBase, ValuesBase as ValuesBase
from .elements import (
    BinaryExpression as BinaryExpression,
    BindParameter as BindParameter,
    BooleanClauseList as BooleanClauseList,
    Case as Case,
    Cast as Cast,
    ClauseElement as ClauseElement,
    ClauseList as ClauseList,
    CollectionAggregate as CollectionAggregate,
    ColumnClause as ColumnClause,
    ColumnElement as ColumnElement,
    Extract as Extract,
    False_ as False_,
    FunctionFilter as FunctionFilter,
    Grouping as Grouping,
    Label as Label,
    Null as Null,
    Over as Over,
    ReleaseSavepointClause as ReleaseSavepointClause,
    RollbackToSavepointClause as RollbackToSavepointClause,
    SavepointClause as SavepointClause,
    TextClause as TextClause,
    True_ as True_,
    Tuple as Tuple,
    TypeClause as TypeClause,
    TypeCoerce as TypeCoerce,
    UnaryExpression as UnaryExpression,
    WithinGroup as WithinGroup,
    _truncated_label as _truncated_label,
    between as between,
    collate as collate,
    literal as literal,
    literal_column as literal_column,
    not_ as not_,
    outparam as outparam,
    quoted_name as quoted_name,
)
from .functions import Function as Function, FunctionElement as FunctionElement, func as func, modifier as modifier
from .lambdas import LambdaElement as LambdaElement, StatementLambdaElement as StatementLambdaElement, lambda_stmt as lambda_stmt
from .operators import ColumnOperators as ColumnOperators, Operators as Operators, custom_op as custom_op
from .selectable import (
    CTE as CTE,
    LABEL_STYLE_DEFAULT as LABEL_STYLE_DEFAULT,
    LABEL_STYLE_DISAMBIGUATE_ONLY as LABEL_STYLE_DISAMBIGUATE_ONLY,
    LABEL_STYLE_NONE as LABEL_STYLE_NONE,
    LABEL_STYLE_TABLENAME_PLUS_COL as LABEL_STYLE_TABLENAME_PLUS_COL,
    Alias as Alias,
    AliasedReturnsRows as AliasedReturnsRows,
    CompoundSelect as CompoundSelect,
    Exists as Exists,
    FromClause as FromClause,
    FromGrouping as FromGrouping,
    GenerativeSelect as GenerativeSelect,
    HasCTE as HasCTE,
    HasPrefixes as HasPrefixes,
    HasSuffixes as HasSuffixes,
    Join as Join,
    Lateral as Lateral,
    ReturnsRows as ReturnsRows,
    ScalarSelect as ScalarSelect,
    Select as Select,
    Selectable as Selectable,
    SelectBase as SelectBase,
    Subquery as Subquery,
    TableClause as TableClause,
    TableSample as TableSample,
    TableValuedAlias as TableValuedAlias,
    TextAsFrom as TextAsFrom,
    TextualSelect as TextualSelect,
    Values as Values,
    subquery as subquery,
)
from .traversals import CacheKey as CacheKey
from .visitors import Visitable as Visitable

__all__ = [
    "Alias",
    "AliasedReturnsRows",
    "any_",
    "all_",
    "CacheKey",
    "ClauseElement",
    "ColumnCollection",
    "ColumnElement",
    "CompoundSelect",
    "Delete",
    "FromClause",
    "Insert",
    "Join",
    "Lateral",
    "LambdaElement",
    "StatementLambdaElement",
    "Select",
    "Selectable",
    "TableClause",
    "TableValuedAlias",
    "Update",
    "Values",
    "alias",
    "and_",
    "asc",
    "between",
    "bindparam",
    "case",
    "cast",
    "column",
    "custom_op",
    "cte",
    "delete",
    "desc",
    "distinct",
    "except_",
    "except_all",
    "exists",
    "extract",
    "func",
    "modifier",
    "collate",
    "insert",
    "intersect",
    "intersect_all",
    "join",
    "label",
    "lateral",
    "lambda_stmt",
    "literal",
    "literal_column",
    "not_",
    "null",
    "nulls_first",
    "nulls_last",
    "or_",
    "outparam",
    "outerjoin",
    "over",
    "select",
    "table",
    "text",
    "tuple_",
    "type_coerce",
    "quoted_name",
    "union",
    "union_all",
    "update",
    "quoted_name",
    "within_group",
    "Subquery",
    "TableSample",
    "tablesample",
    "values",
]

all_: Any
any_: Any
and_: Any
alias: Any
tablesample: Any
lateral: Any
or_: Any
bindparam: Any
select: Any
text: Any
table: Any
column: Any
over: Any
within_group: Any
label: Any
case: Any
cast: Any
cte: Any
values: Any
extract: Any
tuple_: Any
except_: Any
except_all: Any
intersect: Any
intersect_all: Any
union: Any
union_all: Any
exists: Any
nulls_first: Any
nullsfirst: Any
nulls_last: Any
nullslast: Any
asc: Any
desc: Any
distinct: Any
type_coerce: Any
true: Any
false: Any
null: Any
join: Any
outerjoin: Any
insert: Any
update: Any
delete: Any
funcfilter: Any



from typing import Any

def cache(fn, self, con, *args, **kw): ...

class Inspector:
    def __init__(self, bind): ...
    @classmethod
    def from_engine(cls, bind): ...
    @property
    def default_schema_name(self): ...
    def get_schema_names(self): ...
    def get_table_names(self, schema: Any | None = ...): ...
    def has_table(self, table_name, schema: Any | None = ...): ...
    def has_sequence(self, sequence_name, schema: Any | None = ...): ...
    def get_sorted_table_and_fkc_names(self, schema: Any | None = ...): ...
    def get_temp_table_names(self): ...
    def get_temp_view_names(self): ...
    def get_table_options(self, table_name, schema: Any | None = ..., **kw): ...
    def get_view_names(self, schema: Any | None = ...): ...
    def get_sequence_names(self, schema: Any | None = ...): ...
    def get_view_definition(self, view_name, schema: Any | None = ...): ...
    def get_columns(self, table_name, schema: Any | None = ..., **kw): ...
    def get_pk_constraint(self, table_name, schema: Any | None = ..., **kw): ...
    def get_foreign_keys(self, table_name, schema: Any | None = ..., **kw): ...
    def get_indexes(self, table_name, schema: Any | None = ..., **kw): ...
    def get_unique_constraints(self, table_name, schema: Any | None = ..., **kw): ...
    def get_table_comment(self, table_name, schema: Any | None = ..., **kw): ...
    def get_check_constraints(self, table_name, schema: Any | None = ..., **kw): ...
    def reflecttable(self, *args, **kwargs): ...
    def reflect_table(
        self, table, include_columns, exclude_columns=..., resolve_fks: bool = ..., _extend_on: Any | None = ...
    ) -> None: ...



from collections.abc import MutableMapping
from typing import Any

from .. import util
from ..util import HasMemoized, hybridmethod, memoized_property
from . import roles
from .traversals import (
    HasCacheKey as HasCacheKey,
    HasCopyInternals as HasCopyInternals,
    MemoizedHasCacheKey as MemoizedHasCacheKey,
)
from .visitors import ClauseVisitor

coercions: Any
elements: Any
type_api: Any
PARSE_AUTOCOMMIT: Any
NO_ARG: Any

class Immutable:
    def unique_params(self, *optionaldict, **kwargs) -> None: ...
    def params(self, *optionaldict, **kwargs) -> None: ...

class SingletonConstant(Immutable):
    def __new__(cls, *arg, **kw): ...

class _DialectArgView(MutableMapping[Any, Any]):
    obj: Any
    def __init__(self, obj) -> None: ...
    def __getitem__(self, key): ...
    def __setitem__(self, key, value) -> None: ...
    def __delitem__(self, key) -> None: ...
    def __len__(self): ...
    def __iter__(self): ...

class _DialectArgDict(MutableMapping[Any, Any]):
    def __init__(self) -> None: ...
    def __len__(self): ...
    def __iter__(self): ...
    def __getitem__(self, key): ...
    def __setitem__(self, key, value) -> None: ...
    def __delitem__(self, key) -> None: ...

class DialectKWArgs:
    @classmethod
    def argument_for(cls, dialect_name, argument_name, default) -> None: ...
    @memoized_property
    def dialect_kwargs(self): ...
    @property
    def kwargs(self): ...
    @memoized_property
    def dialect_options(self): ...

class CompileState:
    plugins: Any
    @classmethod
    def create_for_statement(cls, statement, compiler, **kw): ...
    statement: Any
    def __init__(self, statement, compiler, **kw) -> None: ...
    @classmethod
    def get_plugin_class(cls, statement): ...
    @classmethod
    def plugin_for(cls, plugin_name, visit_name): ...

class Generative(HasMemoized): ...
class InPlaceGenerative(HasMemoized): ...
class HasCompileState(Generative): ...

class _MetaOptions(type):
    def __init__(cls, classname, bases, dict_) -> None: ...
    def __add__(self, other): ...

class Options:
    def __init__(self, **kw) -> None: ...
    def __add__(self, other): ...
    def __eq__(self, other): ...
    @classmethod
    def isinstance(cls, klass): ...
    @hybridmethod
    def add_to_element(self, name, value): ...
    @classmethod
    def safe_merge(cls, other): ...
    @classmethod
    def from_execution_options(cls, key, attrs, exec_options, statement_exec_options): ...

class CacheableOptions(Options, HasCacheKey): ...

class ExecutableOption(HasCopyInternals):
    __visit_name__: str

class Executable(roles.StatementRole, Generative):
    supports_execution: bool
    is_select: bool
    is_update: bool
    is_insert: bool
    is_text: bool
    is_delete: bool
    is_dml: bool
    def options(self, *options) -> None: ...
    def execution_options(self, **kw) -> None: ...
    def get_execution_options(self): ...
    def execute(self, *multiparams, **params): ...
    def scalar(self, *multiparams, **params): ...
    @property
    def bind(self): ...

class prefix_anon_map(dict[Any, Any]):
    def __missing__(self, key): ...

class SchemaEventTarget: ...

class SchemaVisitor(ClauseVisitor):
    __traverse_options__: Any

class ColumnCollection:
    def __init__(self, columns: Any | None = ...) -> None: ...
    def keys(self): ...
    def values(self): ...
    def items(self): ...
    def __bool__(self): ...
    def __len__(self): ...
    def __iter__(self): ...
    def __getitem__(self, key): ...
    def __getattr__(self, key): ...
    def __contains__(self, key): ...
    def compare(self, other): ...
    def __eq__(self, other): ...
    def get(self, key, default: Any | None = ...): ...
    def __setitem__(self, key, value) -> None: ...
    def __delitem__(self, key) -> None: ...
    def __setattr__(self, key, obj) -> None: ...
    def clear(self) -> None: ...
    def remove(self, column) -> None: ...
    def update(self, iter_) -> None: ...
    __hash__: Any
    def add(self, column, key: Any | None = ...) -> None: ...
    def contains_column(self, col): ...
    def as_immutable(self): ...
    def corresponding_column(self, column, require_embedded: bool = ...): ...

class DedupeColumnCollection(ColumnCollection):
    def add(self, column, key: Any | None = ...) -> None: ...
    def extend(self, iter_) -> None: ...
    def remove(self, column) -> None: ...
    def replace(self, column) -> None: ...

class ImmutableColumnCollection(util.ImmutableContainer, ColumnCollection):
    def __init__(self, collection) -> None: ...
    add: Any
    extend: Any
    remove: Any

class ColumnSet(util.ordered_column_set):
    def contains_column(self, col): ...
    def extend(self, cols) -> None: ...
    def __add__(self, other): ...
    def __eq__(self, other): ...
    def __hash__(self): ...



import typing

from .array import ARRAY as ARRAY, All as All, Any as Any, array as array
from .base import (
    BIGINT as BIGINT,
    BIT as BIT,
    BOOLEAN as BOOLEAN,
    BYTEA as BYTEA,
    CHAR as CHAR,
    CIDR as CIDR,
    DATE as DATE,
    DOUBLE_PRECISION as DOUBLE_PRECISION,
    ENUM as ENUM,
    FLOAT as FLOAT,
    INET as INET,
    INTEGER as INTEGER,
    INTERVAL as INTERVAL,
    MACADDR as MACADDR,
    MONEY as MONEY,
    NUMERIC as NUMERIC,
    OID as OID,
    REAL as REAL,
    REGCLASS as REGCLASS,
    SMALLINT as SMALLINT,
    TEXT as TEXT,
    TIME as TIME,
    TIMESTAMP as TIMESTAMP,
    TSVECTOR as TSVECTOR,
    UUID as UUID,
    VARCHAR as VARCHAR,
    CreateEnumType as CreateEnumType,
    DropEnumType as DropEnumType,
)
from .dml import Insert as Insert, insert as insert
from .ext import ExcludeConstraint as ExcludeConstraint, aggregate_order_by as aggregate_order_by, array_agg as array_agg
from .hstore import HSTORE as HSTORE, hstore as hstore
from .json import JSON as JSON, JSONB as JSONB
from .ranges import (
    DATERANGE as DATERANGE,
    INT4RANGE as INT4RANGE,
    INT8RANGE as INT8RANGE,
    NUMRANGE as NUMRANGE,
    TSRANGE as TSRANGE,
    TSTZRANGE as TSTZRANGE,
)

__all__ = (
    "INTEGER",
    "BIGINT",
    "SMALLINT",
    "VARCHAR",
    "CHAR",
    "TEXT",
    "NUMERIC",
    "FLOAT",
    "REAL",
    "INET",
    "CIDR",
    "UUID",
    "BIT",
    "MACADDR",
    "MONEY",
    "OID",
    "REGCLASS",
    "DOUBLE_PRECISION",
    "TIMESTAMP",
    "TIME",
    "DATE",
    "BYTEA",
    "BOOLEAN",
    "INTERVAL",
    "ARRAY",
    "ENUM",
    "dialect",
    "array",
    "HSTORE",
    "hstore",
    "INT4RANGE",
    "INT8RANGE",
    "NUMRANGE",
    "DATERANGE",
    "TSVECTOR",
    "TSRANGE",
    "TSTZRANGE",
    "JSON",
    "JSONB",
    "Any",
    "All",
    "DropEnumType",
    "CreateEnumType",
    "ExcludeConstraint",
    "aggregate_order_by",
    "array_agg",
    "insert",
    "Insert",
)

dialect: typing.Any



from setuptools import setup

name = "types-SQLAlchemy"
description = "Typing stubs for SQLAlchemy"
long_description = '''
## Typing stubs for SQLAlchemy

This is a PEP 561 type stub package for the `SQLAlchemy` package.
It can be used by type-checking tools like mypy, PyCharm, pytype etc. to check code
that uses `SQLAlchemy`. The source for this package can be found at
https://github.com/python/typeshed/tree/master/stubs/SQLAlchemy. All fixes for
types and metadata should be contributed there.

The `sqlalchemy-stubs` package is an alternative to this package and also includes a mypy plugin for more precise types.

See https://github.com/python/typeshed/blob/master/README.md for more details.
This package was generated from typeshed commit `2ad8db51ada34bd7d971781feb438bd135261c7b`.
'''.lstrip()

setup(name=name,
      version="1.4.6",
      description=description,
      long_description=long_description,
      long_description_content_type="text/markdown",
      url="https://github.com/python/typeshed",
      install_requires=[],
      packages=['sqlalchemy-stubs'],
      package_data={'sqlalchemy-stubs': ['__init__.pyi', 'cimmutabledict.pyi', 'connectors/__init__.pyi', 'connectors/mxodbc.pyi', 'connectors/pyodbc.pyi', 'cresultproxy.pyi', 'databases/__init__.pyi', 'dialects/__init__.pyi', 'dialects/firebird/__init__.pyi', 'dialects/firebird/base.pyi', 'dialects/firebird/fdb.pyi', 'dialects/firebird/kinterbasdb.pyi', 'dialects/mssql/__init__.pyi', 'dialects/mssql/base.pyi', 'dialects/mssql/information_schema.pyi', 'dialects/mssql/json.pyi', 'dialects/mssql/mxodbc.pyi', 'dialects/mssql/provision.pyi', 'dialects/mssql/pymssql.pyi', 'dialects/mssql/pyodbc.pyi', 'dialects/mysql/__init__.pyi', 'dialects/mysql/aiomysql.pyi', 'dialects/mysql/asyncmy.pyi', 'dialects/mysql/base.pyi', 'dialects/mysql/cymysql.pyi', 'dialects/mysql/dml.pyi', 'dialects/mysql/enumerated.pyi', 'dialects/mysql/expression.pyi', 'dialects/mysql/json.pyi', 'dialects/mysql/mariadb.pyi', 'dialects/mysql/mariadbconnector.pyi', 'dialects/mysql/mysqlconnector.pyi', 'dialects/mysql/mysqldb.pyi', 'dialects/mysql/oursql.pyi', 'dialects/mysql/provision.pyi', 'dialects/mysql/pymysql.pyi', 'dialects/mysql/pyodbc.pyi', 'dialects/mysql/reflection.pyi', 'dialects/mysql/reserved_words.pyi', 'dialects/mysql/types.pyi', 'dialects/oracle/__init__.pyi', 'dialects/oracle/base.pyi', 'dialects/oracle/cx_oracle.pyi', 'dialects/oracle/provision.pyi', 'dialects/postgresql/__init__.pyi', 'dialects/postgresql/array.pyi', 'dialects/postgresql/asyncpg.pyi', 'dialects/postgresql/base.pyi', 'dialects/postgresql/dml.pyi', 'dialects/postgresql/ext.pyi', 'dialects/postgresql/hstore.pyi', 'dialects/postgresql/json.pyi', 'dialects/postgresql/pg8000.pyi', 'dialects/postgresql/provision.pyi', 'dialects/postgresql/psycopg2.pyi', 'dialects/postgresql/psycopg2cffi.pyi', 'dialects/postgresql/pygresql.pyi', 'dialects/postgresql/pypostgresql.pyi', 'dialects/postgresql/ranges.pyi', 'dialects/sqlite/__init__.pyi', 'dialects/sqlite/aiosqlite.pyi', 'dialects/sqlite/base.pyi', 'dialects/sqlite/dml.pyi', 'dialects/sqlite/json.pyi', 'dialects/sqlite/provision.pyi', 'dialects/sqlite/pysqlcipher.pyi', 'dialects/sqlite/pysqlite.pyi', 'dialects/sybase/__init__.pyi', 'dialects/sybase/base.pyi', 'dialects/sybase/mxodbc.pyi', 'dialects/sybase/pyodbc.pyi', 'dialects/sybase/pysybase.pyi', 'engine/__init__.pyi', 'engine/base.pyi', 'engine/characteristics.pyi', 'engine/create.pyi', 'engine/cursor.pyi', 'engine/default.pyi', 'engine/events.pyi', 'engine/interfaces.pyi', 'engine/mock.pyi', 'engine/reflection.pyi', 'engine/result.pyi', 'engine/row.pyi', 'engine/strategies.pyi', 'engine/url.pyi', 'engine/util.pyi', 'event/__init__.pyi', 'event/api.pyi', 'event/attr.pyi', 'event/base.pyi', 'event/legacy.pyi', 'event/registry.pyi', 'events.pyi', 'exc.pyi', 'ext/__init__.pyi', 'ext/associationproxy.pyi', 'ext/asyncio/__init__.pyi', 'ext/asyncio/base.pyi', 'ext/asyncio/engine.pyi', 'ext/asyncio/events.pyi', 'ext/asyncio/exc.pyi', 'ext/asyncio/result.pyi', 'ext/asyncio/scoping.pyi', 'ext/asyncio/session.pyi', 'ext/automap.pyi', 'ext/baked.pyi', 'ext/compiler.pyi', 'ext/declarative/__init__.pyi', 'ext/declarative/extensions.pyi', 'ext/horizontal_shard.pyi', 'ext/hybrid.pyi', 'ext/indexable.pyi', 'ext/instrumentation.pyi', 'ext/mutable.pyi', 'ext/mypy/__init__.pyi', 'ext/mypy/apply.pyi', 'ext/mypy/decl_class.pyi', 'ext/mypy/infer.pyi', 'ext/mypy/names.pyi', 'ext/mypy/plugin.pyi', 'ext/mypy/util.pyi', 'ext/orderinglist.pyi', 'ext/serializer.pyi', 'future/__init__.pyi', 'future/engine.pyi', 'future/orm/__init__.pyi', 'inspection.pyi', 'log.pyi', 'orm/__init__.pyi', 'orm/attributes.pyi', 'orm/base.pyi', 'orm/clsregistry.pyi', 'orm/collections.pyi', 'orm/context.pyi', 'orm/decl_api.pyi', 'orm/decl_base.pyi', 'orm/dependency.pyi', 'orm/descriptor_props.pyi', 'orm/dynamic.pyi', 'orm/evaluator.pyi', 'orm/events.pyi', 'orm/exc.pyi', 'orm/identity.pyi', 'orm/instrumentation.pyi', 'orm/interfaces.pyi', 'orm/loading.pyi', 'orm/mapper.pyi', 'orm/path_registry.pyi', 'orm/persistence.pyi', 'orm/properties.pyi', 'orm/query.pyi', 'orm/relationships.pyi', 'orm/scoping.pyi', 'orm/session.pyi', 'orm/state.pyi', 'orm/strategies.pyi', 'orm/strategy_options.pyi', 'orm/sync.pyi', 'orm/unitofwork.pyi', 'orm/util.pyi', 'pool/__init__.pyi', 'pool/base.pyi', 'pool/dbapi_proxy.pyi', 'pool/events.pyi', 'pool/impl.pyi', 'processors.pyi', 'schema.pyi', 'sql/__init__.pyi', 'sql/annotation.pyi', 'sql/base.pyi', 'sql/coercions.pyi', 'sql/compiler.pyi', 'sql/crud.pyi', 'sql/ddl.pyi', 'sql/default_comparator.pyi', 'sql/dml.pyi', 'sql/elements.pyi', 'sql/events.pyi', 'sql/expression.pyi', 'sql/functions.pyi', 'sql/lambdas.pyi', 'sql/naming.pyi', 'sql/operators.pyi', 'sql/roles.pyi', 'sql/schema.pyi', 'sql/selectable.pyi', 'sql/sqltypes.pyi', 'sql/traversals.pyi', 'sql/type_api.pyi', 'sql/util.pyi', 'sql/visitors.pyi', 'testing/__init__.pyi', 'testing/assertions.pyi', 'testing/assertsql.pyi', 'testing/asyncio.pyi', 'testing/config.pyi', 'testing/engines.pyi', 'testing/entities.pyi', 'testing/exclusions.pyi', 'testing/fixtures.pyi', 'testing/mock.pyi', 'testing/pickleable.pyi', 'testing/plugin/__init__.pyi', 'testing/plugin/bootstrap.pyi', 'testing/plugin/plugin_base.pyi', 'testing/plugin/pytestplugin.pyi', 'testing/plugin/reinvent_fixtures_py2k.pyi', 'testing/profiling.pyi', 'testing/provision.pyi', 'testing/requirements.pyi', 'testing/schema.pyi', 'testing/util.pyi', 'testing/warnings.pyi', 'types.pyi', 'util/__init__.pyi', 'util/_collections.pyi', 'util/_compat_py3k.pyi', 'util/_concurrency_py3k.pyi', 'util/_preloaded.pyi', 'util/compat.pyi', 'util/concurrency.pyi', 'util/deprecations.pyi', 'util/langhelpers.pyi', 'util/queue.pyi', 'util/topological.pyi', 'METADATA.toml']},
      license="Apache-2.0 license",
      classifiers=[
          "License :: OSI Approved :: Apache Software License",
          "Typing :: Stubs Only",
      ]
)



class Requirements: ...

class SuiteRequirements(Requirements):
    @property
    def create_table(self): ...
    @property
    def drop_table(self): ...
    @property
    def table_ddl_if_exists(self): ...
    @property
    def index_ddl_if_exists(self): ...
    @property
    def foreign_keys(self): ...
    @property
    def table_value_constructor(self): ...
    @property
    def standard_cursor_sql(self): ...
    @property
    def on_update_cascade(self): ...
    @property
    def non_updating_cascade(self): ...
    @property
    def deferrable_fks(self): ...
    @property
    def on_update_or_deferrable_fks(self): ...
    @property
    def queue_pool(self): ...
    @property
    def self_referential_foreign_keys(self): ...
    @property
    def foreign_key_ddl(self): ...
    @property
    def named_constraints(self): ...
    @property
    def implicitly_named_constraints(self): ...
    @property
    def subqueries(self): ...
    @property
    def offset(self): ...
    @property
    def bound_limit_offset(self): ...
    @property
    def sql_expression_limit_offset(self): ...
    @property
    def parens_in_union_contained_select_w_limit_offset(self): ...
    @property
    def parens_in_union_contained_select_wo_limit_offset(self): ...
    @property
    def boolean_col_expressions(self): ...
    @property
    def nullable_booleans(self): ...
    @property
    def nullsordering(self): ...
    @property
    def standalone_binds(self): ...
    @property
    def standalone_null_binds_whereclause(self): ...
    @property
    def intersect(self): ...
    @property
    def except_(self): ...
    @property
    def window_functions(self): ...
    @property
    def ctes(self): ...
    @property
    def ctes_with_update_delete(self): ...
    @property
    def ctes_on_dml(self): ...
    @property
    def autoincrement_insert(self): ...
    @property
    def fetch_rows_post_commit(self): ...
    @property
    def group_by_complex_expression(self): ...
    @property
    def sane_rowcount(self): ...
    @property
    def sane_multi_rowcount(self): ...
    @property
    def sane_rowcount_w_returning(self): ...
    @property
    def empty_inserts(self): ...
    @property
    def empty_inserts_executemany(self): ...
    @property
    def insert_from_select(self): ...
    @property
    def full_returning(self): ...
    @property
    def insert_executemany_returning(self): ...
    @property
    def returning(self): ...
    @property
    def tuple_in(self): ...
    @property
    def tuple_in_w_empty(self): ...
    @property
    def duplicate_names_in_cursor_description(self): ...
    @property
    def denormalized_names(self): ...
    @property
    def multivalues_inserts(self): ...
    @property
    def implements_get_lastrowid(self): ...
    @property
    def emulated_lastrowid(self): ...
    @property
    def emulated_lastrowid_even_with_sequences(self): ...
    @property
    def dbapi_lastrowid(self): ...
    @property
    def views(self): ...
    @property
    def schemas(self): ...
    @property
    def cross_schema_fk_reflection(self): ...
    @property
    def foreign_key_constraint_name_reflection(self): ...
    @property
    def implicit_default_schema(self): ...
    @property
    def default_schema_name_switch(self): ...
    @property
    def server_side_cursors(self): ...
    @property
    def sequences(self): ...
    @property
    def no_sequences(self): ...
    @property
    def sequences_optional(self): ...
    @property
    def supports_lastrowid(self): ...
    @property
    def no_lastrowid_support(self): ...
    @property
    def reflects_pk_names(self): ...
    @property
    def table_reflection(self): ...
    @property
    def reflect_tables_no_columns(self): ...
    @property
    def comment_reflection(self): ...
    @property
    def view_column_reflection(self): ...
    @property
    def view_reflection(self): ...
    @property
    def schema_reflection(self): ...
    @property
    def primary_key_constraint_reflection(self): ...
    @property
    def foreign_key_constraint_reflection(self): ...
    @property
    def foreign_key_constraint_option_reflection_ondelete(self): ...
    @property
    def fk_constraint_option_reflection_ondelete_restrict(self): ...
    @property
    def fk_constraint_option_reflection_ondelete_noaction(self): ...
    @property
    def foreign_key_constraint_option_reflection_onupdate(self): ...
    @property
    def fk_constraint_option_reflection_onupdate_restrict(self): ...
    @property
    def temp_table_reflection(self): ...
    @property
    def temp_table_reflect_indexes(self): ...
    @property
    def temp_table_names(self): ...
    @property
    def temporary_tables(self): ...
    @property
    def temporary_views(self): ...
    @property
    def index_reflection(self): ...
    @property
    def index_reflects_included_columns(self): ...
    @property
    def indexes_with_ascdesc(self): ...
    @property
    def indexes_with_expressions(self): ...
    @property
    def unique_constraint_reflection(self): ...
    @property
    def check_constraint_reflection(self): ...
    @property
    def duplicate_key_raises_integrity_error(self): ...
    @property
    def unbounded_varchar(self): ...
    @property
    def unicode_data(self): ...
    @property
    def unicode_ddl(self): ...
    @property
    def symbol_names_w_double_quote(self): ...
    @property
    def datetime_literals(self): ...
    @property
    def datetime(self): ...
    @property
    def datetime_microseconds(self): ...
    @property
    def timestamp_microseconds(self): ...
    @property
    def datetime_historic(self): ...
    @property
    def date(self): ...
    @property
    def date_coerces_from_datetime(self): ...
    @property
    def date_historic(self): ...
    @property
    def time(self): ...
    @property
    def time_microseconds(self): ...
    @property
    def binary_comparisons(self): ...
    @property
    def binary_literals(self): ...
    @property
    def autocommit(self): ...
    @property
    def isolation_level(self): ...
    def get_isolation_levels(self, config) -> None: ...
    @property
    def json_type(self): ...
    @property
    def json_array_indexes(self): ...
    @property
    def json_index_supplementary_unicode_element(self): ...
    @property
    def legacy_unconditional_json_extract(self): ...
    @property
    def precision_numerics_general(self): ...
    @property
    def precision_numerics_enotation_small(self): ...
    @property
    def precision_numerics_enotation_large(self): ...
    @property
    def precision_numerics_many_significant_digits(self): ...
    @property
    def cast_precision_numerics_many_significant_digits(self): ...
    @property
    def implicit_decimal_binds(self): ...
    @property
    def nested_aggregates(self): ...
    @property
    def recursive_fk_cascade(self): ...
    @property
    def precision_numerics_retains_significant_digits(self): ...
    @property
    def infinity_floats(self): ...
    @property
    def precision_generic_float_type(self): ...
    @property
    def floats_to_four_decimals(self): ...
    @property
    def fetch_null_from_numeric(self): ...
    @property
    def text_type(self): ...
    @property
    def empty_strings_varchar(self): ...
    @property
    def empty_strings_text(self): ...
    @property
    def expressions_against_unbounded_text(self): ...
    @property
    def selectone(self): ...
    @property
    def savepoints(self): ...
    @property
    def two_phase_transactions(self): ...
    @property
    def update_from(self): ...
    @property
    def delete_from(self): ...
    @property
    def update_where_target_in_subquery(self): ...
    @property
    def mod_operator_as_percent_sign(self): ...
    @property
    def percent_schema_names(self): ...
    @property
    def order_by_col_from_union(self): ...
    @property
    def order_by_label_with_expression(self): ...
    @property
    def order_by_collation(self): ...
    def get_order_by_collation(self, config) -> None: ...
    @property
    def unicode_connections(self): ...
    @property
    def graceful_disconnects(self): ...
    @property
    def independent_connections(self): ...
    @property
    def skip_mysql_on_windows(self): ...
    @property
    def ad_hoc_engines(self): ...
    @property
    def no_windows(self): ...
    @property
    def timing_intensive(self): ...
    @property
    def memory_intensive(self): ...
    @property
    def threading_with_mock(self): ...
    @property
    def sqlalchemy2_stubs(self): ...
    @property
    def python2(self): ...
    @property
    def python3(self): ...
    @property
    def pep520(self): ...
    @property
    def insert_order_dicts(self): ...
    @property
    def python36(self): ...
    @property
    def python37(self): ...
    @property
    def dataclasses(self): ...
    @property
    def python38(self): ...
    @property
    def cpython(self): ...
    @property
    def patch_library(self): ...
    @property
    def non_broken_pickle(self): ...
    @property
    def predictable_gc(self): ...
    @property
    def no_coverage(self): ...
    @property
    def sqlite(self): ...
    @property
    def cextensions(self): ...
    @property
    def async_dialect(self): ...
    @property
    def greenlet(self): ...
    @property
    def computed_columns(self): ...
    @property
    def computed_columns_stored(self): ...
    @property
    def computed_columns_virtual(self): ...
    @property
    def computed_columns_default_persisted(self): ...
    @property
    def computed_columns_reflect_persisted(self): ...
    @property
    def supports_distinct_on(self): ...
    @property
    def supports_is_distinct_from(self): ...
    @property
    def identity_columns(self): ...
    @property
    def identity_columns_standard(self): ...
    @property
    def regexp_match(self): ...
    @property
    def regexp_replace(self): ...
    @property
    def fetch_first(self): ...
    @property
    def fetch_percent(self): ...
    @property
    def fetch_ties(self): ...
    @property
    def fetch_no_order_by(self): ...
    @property
    def fetch_offset_with_options(self): ...
    @property
    def fetch_expression(self): ...
    @property
    def autoincrement_without_sequence(self): ...
    @property
    def generic_classes(self): ...



from typing import Any

from ...sql import sqltypes
from .types import _StringType

class ENUM(sqltypes.NativeForEmulated, sqltypes.Enum, _StringType):  # type: ignore[misc]
    __visit_name__: str
    native_enum: bool
    def __init__(self, *enums, **kw) -> None: ...
    @classmethod
    def adapt_emulated_to_native(cls, impl, **kw): ...

class SET(_StringType):
    __visit_name__: str
    retrieve_as_bitwise: Any
    values: Any
    def __init__(self, *values, **kw) -> None: ...
    def column_expression(self, colexpr): ...
    def result_processor(self, dialect, coltype): ...
    def bind_processor(self, dialect): ...
    def adapt(self, impltype, **kw): ...



from typing import Any

class MutableBase:
    @classmethod
    def coerce(cls, key, value) -> None: ...

class Mutable(MutableBase):
    def changed(self) -> None: ...
    @classmethod
    def associate_with_attribute(cls, attribute) -> None: ...
    @classmethod
    def associate_with(cls, sqltype) -> None: ...
    @classmethod
    def as_mutable(cls, sqltype): ...

class MutableComposite(MutableBase):
    def changed(self) -> None: ...

class MutableDict(Mutable, dict[Any, Any]):
    def __setitem__(self, key, value) -> None: ...
    def setdefault(self, key, value): ...
    def __delitem__(self, key) -> None: ...
    def update(self, *a, **kw) -> None: ...
    def pop(self, *arg): ...
    def popitem(self): ...
    def clear(self) -> None: ...
    @classmethod
    def coerce(cls, key, value): ...

class MutableList(Mutable, list[Any]):
    def __reduce_ex__(self, proto): ...
    def __setitem__(self, index, value) -> None: ...
    def __setslice__(self, start, end, value) -> None: ...
    def __delitem__(self, index) -> None: ...
    def __delslice__(self, start, end) -> None: ...
    def pop(self, *arg): ...
    def append(self, x) -> None: ...
    def extend(self, x) -> None: ...
    def __iadd__(self, x): ...  # type: ignore[misc]
    def insert(self, i, x) -> None: ...
    def remove(self, i) -> None: ...
    def clear(self) -> None: ...
    def sort(self, **kw) -> None: ...
    def reverse(self) -> None: ...
    @classmethod
    def coerce(cls, index, value): ...

class MutableSet(Mutable, set[Any]):
    def update(self, *arg) -> None: ...
    def intersection_update(self, *arg) -> None: ...
    def difference_update(self, *arg) -> None: ...
    def symmetric_difference_update(self, *arg) -> None: ...
    def __ior__(self, other): ...  # type: ignore[misc]
    def __iand__(self, other): ...  # type: ignore[misc]
    def __ixor__(self, other): ...  # type: ignore[misc]
    def __isub__(self, other): ...  # type: ignore[misc]
    def add(self, elem) -> None: ...
    def remove(self, elem) -> None: ...
    def discard(self, elem) -> None: ...
    def pop(self, *arg): ...
    def clear(self) -> None: ...
    @classmethod
    def coerce(cls, index, value): ...
    def __reduce_ex__(self, proto): ...



from typing import Any

from . import visitors

join_condition: Any

def find_join_source(clauses, join_to): ...
def find_left_clause_that_matches_given(clauses, join_from): ...
def find_left_clause_to_join_from(clauses, join_to, onclause): ...
def visit_binary_product(fn, expr) -> None: ...
def find_tables(
    clause,
    check_columns: bool = ...,
    include_aliases: bool = ...,
    include_joins: bool = ...,
    include_selects: bool = ...,
    include_crud: bool = ...,
): ...
def unwrap_order_by(clause): ...
def unwrap_label_reference(element): ...
def expand_column_list_from_order_by(collist, order_by): ...
def clause_is_present(clause, search): ...
def tables_from_leftmost(clause) -> None: ...
def surface_selectables(clause) -> None: ...
def surface_selectables_only(clause) -> None: ...
def extract_first_column_annotation(column, annotation_name): ...
def selectables_overlap(left, right): ...
def bind_values(clause): ...

class _repr_base:
    max_chars: Any
    def trunc(self, value): ...

class _repr_row(_repr_base):
    row: Any
    max_chars: Any
    def __init__(self, row, max_chars: int = ...) -> None: ...

class _repr_params(_repr_base):
    params: Any
    ismulti: Any
    batches: Any
    max_chars: Any
    def __init__(self, params, batches, max_chars: int = ..., ismulti: Any | None = ...) -> None: ...

def adapt_criterion_to_null(crit, nulls): ...
def splice_joins(left, right, stop_on: Any | None = ...): ...
def reduce_columns(columns, *clauses, **kw): ...
def criterion_as_pairs(
    expression,
    consider_as_foreign_keys: Any | None = ...,
    consider_as_referenced_keys: Any | None = ...,
    any_operator: bool = ...,
): ...

class ClauseAdapter(visitors.ReplacingExternalTraversal):
    __traverse_options__: Any
    selectable: Any
    include_fn: Any
    exclude_fn: Any
    equivalents: Any
    adapt_on_names: Any
    adapt_from_selectables: Any
    def __init__(
        self,
        selectable,
        equivalents: Any | None = ...,
        include_fn: Any | None = ...,
        exclude_fn: Any | None = ...,
        adapt_on_names: bool = ...,
        anonymize_labels: bool = ...,
        adapt_from_selectables: Any | None = ...,
    ) -> None: ...
    def replace(self, col, _include_singleton_constants: bool = ...): ...

class ColumnAdapter(ClauseAdapter):
    columns: Any
    adapt_required: Any
    allow_label_resolve: Any
    def __init__(
        self,
        selectable,
        equivalents: Any | None = ...,
        adapt_required: bool = ...,
        include_fn: Any | None = ...,
        exclude_fn: Any | None = ...,
        adapt_on_names: bool = ...,
        allow_label_resolve: bool = ...,
        anonymize_labels: bool = ...,
        adapt_from_selectables: Any | None = ...,
    ) -> None: ...
    class _IncludeExcludeMapping:
        parent: Any
        columns: Any
        def __init__(self, parent, columns) -> None: ...
        def __getitem__(self, key): ...
    def wrap(self, adapter): ...
    def traverse(self, obj): ...
    adapt_clause: Any
    adapt_list: Any
    def adapt_check_present(self, col): ...



from typing import Any

from . import util

AssignmentStmt = Any  # from mypy.nodes
NameExpr = Any  # from mypy.nodes
StrExpr = Any  # from mypy.nodes
SemanticAnalyzerPluginInterface = Any  # from mypy.plugin
ProperType = Any  # from mypy.types

def apply_mypy_mapped_attr(
    cls, api: SemanticAnalyzerPluginInterface, item: NameExpr | StrExpr, attributes: list[util.SQLAlchemyAttribute]
) -> None: ...
def re_apply_declarative_assignments(
    cls, api: SemanticAnalyzerPluginInterface, attributes: list[util.SQLAlchemyAttribute]
) -> None: ...
def apply_type_to_mapped_statement(
    api: SemanticAnalyzerPluginInterface,
    stmt: AssignmentStmt,
    lvalue: NameExpr,
    left_hand_explicit_type: ProperType | None,
    python_type_for_type: ProperType | None,
) -> None: ...
def add_additional_orm_attributes(
    cls, api: SemanticAnalyzerPluginInterface, attributes: list[util.SQLAlchemyAttribute]
) -> None: ...
